
/****************************************************************************
 *   testRTree.cc : 
 *   		this file tests the RTree spatial index.
 *   Change Log :
 *		  Sept.20-1997 Initial version,  Liujian Qian
 *
 *   $Id: testRTree.cpp,v 1.5 1999/02/14 03:14:21 qian Exp $
 ***************************************************************************/

#include "gadt_point.h"
#include "gadt_rect.h"
#include "gadt_arc.h"
#include "gadt_polygon.h"
#include "gshapefile.h"
#include "rt.h"

// this function builds a RTree index for the records in
// the given shapefile (suppose its a arc-type shapefile)
Result indexShapefile(const char* sname, const char* output)
{
    GShapeFile	shp;
    GPolygon	*pgn;
    GRect	mbr;
    RTree	rt;
    REntry	e;
    
    // open shapefile
    if (shp.openShapeFile(sname) != GOk) 
	msgFail("can't open shapefile");
    
    cout<<"shape file:"<<shp<<endl;

    // create the rtree index file
    rt.create(output);
    cout<<"rtree "<<output<<" created:"<<rt;
    
    // walk through all the polygons in data file 
    // and insert them into the RTree
    for(int i=0; i<shp.numRecords()-1; i++) {
	if( !(pgn = (GPolygon*) shp.getShapeRec(i)) ) 
	    msgFail("can't get shape record");
	cout<<"."; cout.flush();
	pgn->mbr(mbr);	//get the mbr of the record
	e.setKey(mbr);			//set the mbr as the key
	e.setPointer(RPtr(i*10+1));	//fake the record id
	rt.insert(e);
    }
    
    cout<<"done indexing. RTree="<<rt;
    rt.close();
    
    return GOk;
}

//
int randomIndex(const char* output, int num, int max)
{
    GRect	mbr;
    RTree	rt;
    REntry	e;

    double	xl, yl, xh, yh;
    
    rt.create(output);
    for(int i=0; i<num; i++) {
	xl = random() % max;
	yl = random() % max;
	xh = random() % max;
	yh = random() % max;
	mbr.set( GMIN(xl,xh), GMIN(yl,yh), GMAX(xh,xl), GMAX(yl,yh));
	e.setKey(mbr);
	e.setPointer( RPtr(i+1) );  //arbitrary pointer here.
	cout<<"inserting record: "<<i+1<<endl;
	rt.insert(e);
    }
    cout<<"done indexing random records: rt="<<rt;
    rt.printAll();
    rt.close();
    return 0;
}

Coord xls[] = {75, 75, 125,175,200,175,300,475,450,575,625,650};
Coord yls[] = {525,275,475,380,250,175,150,175,275,450,400,500};
Coord xhs[] = {100,125,225,345,250,225,375,525,500,700,675,725};
Coord yhs[] = {600,350,550,440,300,225,500,425,325,525,626,550};

/*
Coord xls[] = {75, 175,200,300,175,475,450,125,75, 575,625,650};
Coord yls[] = {275,175,250,150,380,175,275,475,525,450,400,500};
Coord xhs[] = {125,225,250,375,345,525,500,225,100,700,675,725};
Coord yhs[] = {350,225,300,500,440,425,325,550,600,525,626,550};
*/
int smallRTree()
{
    GRect	mbr;
    RTree	rt;
    REntry	e;

    double	xl, yl, xh, yh;

    cout<<"page capacity="<<MAX_ETS<<endl;
    
    rt.create("small.idx");
    for(int i=0; i<sizeof(xls)/sizeof(Coord); i++) {
	xl = xls[i], yl = yls[i], xh = xhs[i], yh = yhs[i];

	mbr.set( GMIN(xl,xh), GMIN(yl,yh), GMAX(xh,xl), GMAX(yl,yh));
	e.setKey(mbr);
	e.setPointer( RPtr(i+10000) );  //arbitrary pointer here.
	rt.insert(e);
    }
    cout<<"done indexing random records: rt="<<rt;
    cout<<"----------------------------------------------------------------------\n";
    rt.printAll();
    rt.close();

    cout<<"opening small.idx....\n";
    rt.open("small.idx");
    rt.printAll();
    rt.close();
    
    return 0;
    
}

int queryRTree(const char* name, double xl, double yl, double xh, double yh)
{
    RTree rtree;
    
    rtree.open(name);
    //rtree.printAll();

    //RKey   range(70, 350, 380, 450);
    RKey   range(xl, yl, xh, yh);
    RPred  query(RPred::OVERLAPS, range);

    int	   cur = rtree.search(query);
    REntry *e;
    int i=0;
    
    while( e = rtree.next(cur) ) {
	i++;
	cout<<"find one entry: "<<*e<<endl;
    }

    cout<<"Total number of satisfying entries: "<<i<<endl;
    
    rtree.close();
    
    return 0;
}

