/****************************************************************************
 *   gadt.cc : implementation file for base ADT and ADT info class.
 *   
 *   Change Log :
 *		   06-28-1996 Initial version,  Liujian Qian
 *		   Nov-1996, major revision, LQ	
 *   $Id: gadt.cpp,v 1.10 1999/02/14 03:14:23 qian Exp $
 ***************************************************************************/
#include <iostream.h>

//#include "gfc.h"
#include "gadt_rect.h"

int 		GADT::n_new = 0;
int 		GADT::n_delete = 0;

int		GADTManager::n_adts = 0;
GADTDef*	GADTManager::adts = 0;


void
GADT::statRep   () 
{
    cout<<"GADT::n_new="<<n_new<<" n_delete="
	<<n_delete<<endl; 
}

GADTDef*		
GADTManager::search(GType t)
{
    register GADTDef* def = adts;
    while(def) {
	if (def->type == t) return def;
	else def = def->next;
    }
    return (GADTDef*)0;
}

int GADTManager::registerADT(GADTDef* def)
{
    GAssert(def);
    //GDBG("registering %s", def->name);
    if(!adts) adts = def;
    else {
	GADTDef* d = adts;
	while(d) {
	    if(d->type == def->type) return 0;
	    d = d->next;
	}
	

	def->next = adts;
	adts = def;
    }
    n_adts++;
    return 1;
}

void  GADTManager::removeADT	(GType )
{
    //GDBG("unregistering %s", getName(t));
    //do nothing
}

const char* GADTManager::getName(GType t) 
{
    GADTDef* def = search(t);
    if(def) return def->name;
    else return 0;
}

int	GADTManager::isFixedSize(GType t)
{
    GADTDef* def = search(t);
    if(def) return def->fixed_size;
    else return 0;
}

GADT*	GADTManager::getPtr(GType t)
{
    GADTDef* def = search(t);
    if(def) return def->instance;
    else return 0;
}

GADT*   GADTManager::getNewPtr	(GType t)
{
    GADTDef* def = search(t);
    if(def) return def->instance->clone();
    else return 0;
}
    
bool	GADTManager::isSpatial	(GType t)
{
    GADTDef* def = search(t);
    if(def) return def->spatial;
    else return 0;
}

bool	GADTManager::isLarge 	(GType t)
{
    switch(t) {
    case _Raster:
    case _Grid:
    case _Graph:
    case _Polygon:
    case _Arc:
	return true;
    default:
	return false;
    }
}

bool	GADTManager::isAreal	(GType t)
{
    switch(t) {
    case _Raster:
    case _Grid:
    case _Polygon:
    case _Graph:
	return true;
    default:
	return false;
    }
}

void    GADTManager:: printAll	(ostream& os) 
{
    GADTDef*	def = adts;
    int  count =0;
    while(def) {
	os<<"ADT("<<def->name<<", "<<def->type<<", ";
	os<<(const char*)(def->fixed_size?"fixed size":"no fixed size")<<", ";
	os<<(const char*)(def->spatial?"spatial":"non spatial");
	os<<")\n";
	def = def->next, count++;
    }
    os<<"total #adt: "<<count<<"\n";
}


int	GADTManager::numADTs		()
{
    return n_adts;
}

