/****************************************************************************
 *   gadt_arc.cpp : 
 *   		main file for Arc ADT implemenation.
 *
 *   Change Log :
 *		 June 18, 1998 Initial version,  Liujian Qian
 *
 *   $Id: gadt_arc.cpp,v 1.9 1999/02/08 07:37:11 qian Exp $
 ***************************************************************************/

#include <string.h>
#include "gfc.h"

int _GIArc::cnt;

GArc::GArc() {}

GArc::GArc(const GArc& a){ops = a.ops;}

GArc::~GArc() {}

GArc& GArc::operator=(const GArc& other)
{
    ops = other.ops;
    return *this;
}

uint32 GArc::getPSize() const 
{
    uint32 sz=0;
    sz = GADT::getPSize() + 
	 ops.getPSize();
    return sz;
}

Result GArc::init(void) { return ops.init(); }

Result 
GArc::input(const char* in)
{
    return ops.input(in);
}

Result 
GArc::output(char*& out)
{
  return ops.output(out);
}


Result 
GArc::pack(DataPipe& p)
{
    GADT::pack(p);
    return ops.pack(p);
}


Result 
GArc::unpack( DataPipe& p)
{
    GADT::unpack(p);
    return ops.unpack(p);
}

void
GArc::set(int n_parts, int n_pnts, const int* parts, const Coord* coord)
{
    ops.set(n_parts, n_pnts, parts, coord);
}

Result
GArc::scale      (double xf, double yf, double zf)
{
    return ops.scale(xf,yf,zf);
}

Result
GArc::translate      (double xf, double yf, double zf)
{
    return ops.translate(xf,yf,zf);
}

Result
GArc::rotate	   (double, bool)
{
    return GOk;
}

Result
GArc::transform  (const GMatrix&)
{
    return GOk;
}

Result 
GArc::mbr(GRect& box) const
{
    return ops.mbr(box);
}


Result 
GArc::genMbr(GRect& r)
{
    return ops.calcMbr(r);
}


double  
GArc::length () const
{
    cout<<ops;
    return ops.totalLength();
}

double
GArc::branchLength(int nth) const
{
    return ops.partLength(nth);
}

Result  
GArc::addBranch(int n_pnts, Coord* coo)
{
    ops.addPart(n_pnts, coo);
    return GOk;    
}

int
GArc::getBranch(int nth, Coord*& coords)
{
    return ops.getPart(nth, coords);
}

int GArc::hasLoop() const
{
    //!!! this is bogus... !!!
    //need to figure out how to do it efficiently
    return false;
}

int
GArc::intersects(int n_point, Coord* coo) const 
{
    return ops.intersects(n_point, coo);
}

int
GArc::intersects(const GOps& o) const
{
    return ops.intersects(o);
}

int GArc::intersects (const GSpatial& a) const
{
    GDBG("Arc intersecting %s ", GADTManager::getName(a.type()) );
    
    switch(a.type()) {
    case _Segment:	return _intersects( (const GSegment&)(a) );
	break;
    case _Rect:		return _intersects( (const GRect&) (a) );
	break;
    case _Circle:	return _intersects( (const GCircle&) a );
	break;
    case _Arc:	 	return _intersects( (const GArc&) (a) );
	break;
    case _Polygon:	return _intersects( (const GPolygon&) (a) );
	break;
	
    default:		
	msgFail("Invalid type of operand!");
    }
}    

int
GArc::_intersects (const GSegment& seg_in) const
{
    return ops.intersects(seg_in);
}

int
GArc::_intersects (const GRect&  r) const
{
    return ops.intersects(r); 
}

int
GArc::_intersects (const GCircle& c) const
{        
    return ops.intersects(c);
}


int    
GArc::_intersects (const GArc& other) const 
{
    return ops.intersects(other.ops);
}

int    
GArc::_intersects (const GPolygon& pgn) const
{
    return pgn.intersects(ops);
}

int    
GArc::isInside     (const GRect& other) const
{
    return ops.isInside(other);
}

int    
GArc::isInside     (const GCircle& other) const
{
    return ops.isInside(other);
}



ostream& operator << (ostream& s, const GArc& g)
{
    GPoint pt;
    
    s<<"(n_point="<<g.ops.n_points<< ", n_branches="<<g.ops.n_parts;
    s<<", PSize="<<g.getPSize()<<" length="<<g.length();
    return s<<")"<<endl;
}
