/****************************************************************************
 *   gcircle_adt.cc : 
 *   		Implementation of the circle ADT.
 *		
 *   Change Log :
 *		  Dec-22-1996 Initial version,  Liujian Qian
 *
 *   $Id: gadt_circle.cpp,v 1.6 1998/12/10 00:11:26 qian Exp $
 ***************************************************************************/
#ifdef _WIN32
#include <strstrea.h>
#else
#include <strstream.h>
#endif

#include <math.h>
#include "gfc.h"

int  _GICircle::cnt;


GCircle& 
GCircle::operator=(const GCircle& other)
{
    if(this==&other) return *this;

    cx =other.cx , cy =other.cy, radius = other.radius;
    return *this;
}

uint32  	 
GCircle::getPSize() const
{
    return GADT::getPSize() + sizeof(double) + sizeof(Coord)*2 ;
}


Result 	 
GCircle::pack(DataPipe& p) 
{
    GADT::pack(p);
    if(GEndian::big) {
	double tmp = cx;
	flipDouble(&tmp);
	p.append(CONPTR(&tmp), sizeof(tmp));
	tmp = cy;
	flipDouble(&tmp);
	p.append(CONPTR(&tmp), sizeof(tmp));
	tmp = radius;
	flipDouble(&tmp);
	p.append(CONPTR(&tmp), sizeof(tmp));
    } else {
	p.append(CONPTR(&cx), sizeof(Coord));
	p.append(CONPTR(&cy), sizeof(Coord));
	p.append(CONPTR(&radius), sizeof(double));
    }

    return GOk;
}

Result 	 
GCircle::unpack    (DataPipe& p)
{
    GADT::unpack(p);    
    GAssert(!p.eop());
    p.fetch((char*)&cx, sizeof(Coord));
    p.fetch((char*)&cy, sizeof(Coord));
    p.fetch((char*)&radius, sizeof(double));
    if(GEndian::big) {
	flipDouble(&cx);
	flipDouble(&cy);
	flipDouble(&radius);
    }
    return GOk;
}
    
//
// user format: "cx cy r"
//
Result 	 
GCircle::input   (const char* in)
{
    istrstream is(in, strlen(in));
    is>>cx>>cy>>radius;
    
    return GOk;
}

    
Result 	 
GCircle::output   (char*& out)
{
    bp.extendTo(120);
    bp.recycle();
    char* p = (char*)bp.ptr();
    sprintf(p, " %f %f %f ", cx, cy, radius);
    out = p;

    return GOk;
}


Result
GCircle::scale      (double xf, double yf, double )
{
    cx *= xf, cy *= yf;
    radius *= GMIN(xf, yf);
    return GOk;
}

Result
GCircle::translate      (double xf, double yf, double )
{
    cx += xf, cy += yf;

    return GOk;
}

Result
GCircle::rotate	   (double, bool)
{
    return GOk;
}

Result
GCircle::transform  (const GMatrix&)
{
    return GOk;
}


 
Result 	 
GCircle::mbr	   (GRect& b) const
{
    b.set((Coord)(cx-radius), (Coord)(cy-radius), 
	  (Coord)(cx+radius), (Coord)(cy+radius));

    return GOk;
}

//
// comparison is only made among same ADTs
//
int	
GCircle::cmp	   (const GADT& other) const
{
    MATCHTYPE(other);
    
    if( equals( (GCircle&)other) ) return 0;
    else if(covers( (GCircle&)other) ) return 1;
    else if( ((GSpatial&)other).covers( (const GSpatial&)*this) ) return -1;
    else return UNDEFINED;
}


void   
GCircle::setArea(Coord _cx, Coord _cy, double area)
{
    cx = _cx, cy = _cy;
    radius  = sqrt(area/_PI_);
}


void   
GCircle::setR(double rad) { radius = rad; }


bool 
GCircle::equals (GCircle& other) const
{
    return (cx==other.cx && cy == other.cy && radius == other.radius);
}

int  
GCircle::covers    (const GSpatial& a) const
{
    switch(a.type()) {
    case _Point:	return _covers( (const GPoint&) (a) );
	break;
    case _Rect:    	return _covers( (const GRect&) (a) );
	break;
    case _Segment:	return _covers( (const GSegment&)(a) );
	break;
    case _Circle:	return _covers( (const GCircle&) (a) );
	break;
    case _Arc: 	 	return _covers( (const GArc&) (a) );
	break;
    case _Polygon:	return _covers( (const GPolygon&) (a) );
	break;
	
    default:		msgWarn("Invalid type of operand!");
	break;
    }
    return UNDEFINED;
}

// return true if this circle covers point 'a'. 
int
GCircle::_covers(const GPoint& a) const 
{
    return ( distPnts(a.getX(), a.getY(), cx, cy) < radius);
}


// return true if this circle covers segment 'a'. 
int
GCircle::_covers(const GSegment& a) const 
{
    return ( distPnts(a.getX1(), a.getY1(), cx, cy) < radius &&
	     distPnts(a.getX2(), a.getY2(), cx, cy) < radius );
}

// return true if this circle covers the rect 'a'
int
GCircle::_covers(const GRect& a) const 
{
    return ( distPnts(a.getXL(), a.getYL(), cx, cy) < radius &&
	     distPnts(a.getXH(), a.getYH(), cx, cy) < radius && 
	     distPnts(a.getXL(), a.getYH(), cx, cy) < radius &&
	     distPnts(a.getXH(), a.getYL(), cx, cy) < radius );
}



// return true if this circle covers circle 'a'. 
int
GCircle::_covers(const GCircle& a) const
{
    double dist_2cntrs = distPnts(cx, cy, a.cx, a.cy);
    return ( (radius - dist_2cntrs) >= a.radius );
}

// return 1 if this circle covers a polyline
int 
GCircle::_covers(const GArc& arc) const
{
    return arc.isInside(*this);
}


// return 1 if this cirlce covers a polygon
int
GCircle::_covers(const GPolygon& pgn) const
{
    return pgn.isInside(*this);
}

int	
GCircle::intersects (const GSpatial& a) const
{
    switch(a.type()) {
    case _Segment:	return _intersects( (const GSegment&)(a) );
	break;
    case _Rect:		return _intersects( (const GRect&) (a) );
	break;
    case _Circle:	return _intersects( (const GCircle&) a );
	break;
    case _Arc:		return _intersects( (const GArc&) (a) );
	break;
    case _Polygon:	return _intersects( (const GPolygon&) (a) );
	break;
	
    default:		msgWarn("Invalid type of operand!");
	break;
    }
    return UNDEFINED;
}


int 
GCircle::_intersects (const GSegment& s) const
{
    return s.intersects(*this);
}


// return true if the two circles overlap each other
int
GCircle::_intersects(const GCircle& a) const
{
    double dist_2cntrs = distPnts(cx, cy, a.cx, a.cy);
    return ( (radius + a.radius) < dist_2cntrs );
}

int
GCircle::_intersects (const GRect& r) const
{
    double rad2 = radius * radius;
    Coord xlow = r.getXL();    Coord ylow = r.getYL();
    Coord xhi  = r.getXH();    Coord yhi  = r.getYH();


    // Translate coordinates, placing C at the origin.  
    xhi  -= cx;	yhi -= cy;
    xlow -= cx;	ylow -= cy;
    
    if(xhi < 0) 		// R to left of circle center 
   	if (yhi < 0) 		// R in lower left corner
	    return ((xhi * xhi + yhi * yhi) < rad2);
   	else if (ylow > 0) 	//R in upper left corner 
	    return ((xhi * xhi + ylow * ylow) < rad2);
   	else 			// R due West of circle 
	    return(GABS(xhi) < radius);
    else if (xlow > 0)  	// R to right of circle center
	if (yhi < 0) 		// R in lower right corner
	    return ((xlow * xlow) < rad2);
   	else if (ylow > 0)  	// R in upper right corner
	    return ((xlow * xlow + ylow + ylow) < rad2);
   	else 			// R due East of circle
	    return (xlow < radius);
    else			// R on circle vertical centerline
	if (yhi < 0) 		//R due South of circle
	    return (GABS(yhi) < radius);
   	else if (ylow > 0)  	// R due North of circle
	    return (ylow < radius);
   	else 			// R contains circle centerpoint
     		return(TRUE);
} 	


// check if this circle intersects a polyline.
int 
GCircle::_intersects (const GArc& r) const
{
    return r.intersects(*this);
}


// check if this circle intersects a polygon.
int 
GCircle::_intersects (const GPolygon& r) const
{
    return r.intersects(*this);
}


ostream& operator<<(ostream& s, const GCircle& c)
{
    return s<<"(cx="<<c.cx<<", cy="<<c.cy<<", r="<<c.radius<<")";
}
