/************************************************************************
 *   gpoint_adt.cc : implementation of GFC point ADT  
 *   
 *   Change Log  :
 *		   Spring 1996 Initial version,  Liujian Qian
 *
 *   $Id: gadt_point.cpp,v 1.5 1999/02/16 22:05:27 qian Exp $
 ************************************************************************/

#include <iostream.h>
#ifdef _WIN32
#include <strstrea.h>
#else
#include <strstream.h>
#endif
#include <string.h>
#include <math.h>

#include "gfc.h"

//int pointadt_registered = 
//   GADTManager::registerADT(new GADTDef( _Point, "Point", 1, 1, new GPoint) );

int _GIPoint::cnt;

GPoint& 
GPoint::operator=(const GPoint& pnt)
{
    if(this==&pnt) return *this;
    
    x=pnt.x, y=pnt.y;
    return *this;
}


uint32 
GPoint::getPSize() const
{
    return (GADT::getPSize() +	// base ADT 
	    sizeof(Coord) * 3) ; 	// coord[]
}


Result  
GPoint::mbr(GRect& box) const
{ 
    box.set(x,y,x,y);

    return GOk;
}

int
GPoint::cmp(const GADT& other) const
{ 
    if(other.type()!=_Point)
	return UNDEFINED; 
    else {
	if(x == ((GPoint&)other).x && y == ((GPoint&)other).y
	   && z == ((GPoint&)other).z)
	    return 0;
	else return 1;
    }
}

Result
GPoint::input( const char* in)
{
    GAssert(in);
    istrstream is(in, strlen(in));
    
    is>>x>>y;
    
    return GOk;
}

Result
GPoint::output(char*& out)
{
    bp.recycle();
    char* p = (char*)bp.ptr();
    sprintf(p, " %f %f ", x, y);
    out = p;
    return GOk;
}

Result
GPoint::pack(DataPipe& p) 
{
    double _x = x;
    double _y = y;
    
    GADT::pack(p);

    if(GEndian::big) {
	flipDouble(&_x);
	flipDouble(&_y);
    }
    p.append(CONPTR(&_x), sizeof(Coord));  //x coordinate
    p.append(CONPTR(&_y), sizeof(Coord));  //y coordinate

    return GOk;
}

Result 
GPoint::unpack(DataPipe& p)
{
    double _x, _y;
    
    GADT::unpack(p);
    GAssert(!p.eop());

    p.fetch((char*)&_x, sizeof(Coord));  // get x coordinate
    p.fetch((char*)&_y, sizeof(Coord));  // get y coordinate
    if(GEndian::big) {
	flipDouble(&_x);
	flipDouble(&_y);
    }
    x = _x, y = _y;
    return GOk;
}


Result
GPoint::scale      (double xf, double yf, double zf)
{
    x *= xf, y *= yf, z *= zf;
    return GOk;
}

Result
GPoint::translate      (double xf, double yf, double zf)
{
    x += xf, y += yf, z+= zf;
    return GOk;
}
 
Result
GPoint::rotate	   (double, bool)
{
    return GOk;
}

Result
GPoint::transform  (const GMatrix&)
{
    return GOk;
}

 
double distPnts (Coord x0, Coord y0, Coord x1, Coord y1)
{
    double xd = fabs ( (double)(x0-x1));
    double yd = fabs ( (double)(y0-y1));
    
    if(!xd) return yd;
    else if(!yd) return xd;
    else return hypot(xd , yd);
}

double	
GPoint::distance() const
{
    return hypot(x, y);
}

double 
GPoint::distance (const GPoint& other) const
{
    return distPnts(x, y, other.x, other.y);
}

double 
GPoint::distance (const GSegment& other) const
{
    //let the segment object does hard work
    return other.distance(*this);
}

int
GPoint::isInside(const GRect& a) const
{
    return (x<=a.getXH() && x>=a.getXL() && y<=a.getYH() && y>=a.getYL());
}

int
GPoint::isInside(const GCircle& a) const
{
    return ( distPnts(x, y, a.getCX(), a.getCY()) < a.getR());

}

int
GPoint::isInside(const GTrig& ) const
{
    return 0;
}

int  
GPoint::isInside(const GPolygon& a) const
{
   return a.covers(*this);    
}

void
GPoint::genCircle(GCircle& c, double r)
{
    c.set(x,y,r);
}

bool
GPoint::equals	     (const GPoint& a) const
{
    return (x==a.x && y==a.y && z==a.z);
}

void
GPoint::genSquare(GRect& rect, double width, double height)
{
    rect.set((Coord)(x-width/2), (Coord)(y-height/2), 
	     (Coord)(x+width/2+1), (Coord)(y+height/2+1) );
}


ostream& operator<<(ostream& s, const GPoint& p)
{
    return s<<"("<<p.x<<", "<<p.y<<")";
}



/*************************************************************************/
/******************************* THE  END ********************************/
/*************************************************************************/


