/****************************************************************************
 *   gadt_polygon.cc :  implementation of polygon ADT
 *   
 *   Change Log :
 *		  06-12-1996 Initial version,  Liujian Qian
 *
 *   $Id: gadt_polygon.cpp,v 1.13 1998/12/10 00:11:28 qian Exp $
 ***************************************************************************/
  
#ifdef __GNUC__
#pragma implementation "gadt_polygon.h"
#endif
#ifdef _WIN32
#include <strstrea.h>
#else
#include <strstream.h>
#endif

#include "gfc.h"

int _GIPolygon::cnt;


GPolygon::GPolygon (){}

GPolygon::GPolygon (const GPolygon& a){ops = a.ops;}

GPolygon::~GPolygon (){}

GPolygon& 
GPolygon::operator= (const GPolygon& a)
{
    ops = a.ops;
    return *this;
}

uint32
GPolygon::getPSize() const
{
    uint32 sz;
    
    //do NOT take into account _capacity; it's volatile.
    sz =  GADT::getPSize() +	// base ADT 
	ops.getPSize();
    return sz;
}


Result 
GPolygon::init(void){ return ops.init(); }

Result 
GPolygon::input(const char* in)
{
    return ops.input(in);
}

Result 
GPolygon::output(char*& out)
{
    return ops.output(out);
}

Result 
GPolygon::pack(DataPipe& p)
{
    GADT::pack(p);
    return ops.pack(p);
}


Result 
GPolygon::unpack( DataPipe& p)
{
    GADT::unpack(p);
    return ops.unpack(p);
}


void
GPolygon::set(int n_parts, int n_pnts, const int* parts, const Coord* coord)
{
    ops.set(n_parts, n_pnts, parts, coord);
}


Result
GPolygon::scale      (double xf, double yf, double zf)
{
    return ops.scale(xf,yf,zf);
}

Result
GPolygon::translate      (double xf, double yf, double zf)
{
    return ops.translate(xf,yf,zf);
}

Result
GPolygon::rotate	   (double, bool)
{
    return GOk;
}

Result
GPolygon::transform  (const GMatrix&)
{
    return GOk;
}


// * simply return the attribute _mbr, which should have been calculated
// * upon initial construction of the object or *2Mem conversion routines.
Result 
GPolygon::mbr(GRect& box) const
{
    return ops.mbr(box);
}

Result 
GPolygon::genMbr(GRect& r)
{      
    double xl,yl,xh,yh;
    xl=xh=ops.coord[0], yl=yh=ops.coord[1];

    for(int i=1,x=0,y=0; i<ops.parts[0]; i++) {
	x=i*PolyDim; y=x+1;
	if(xl>ops.coord[x]) xl = ops.coord[x];
	if(xh<ops.coord[x]) xh = ops.coord[x];
	if(yl>ops.coord[y]) yl = ops.coord[y];
	if(yh<ops.coord[y]) yh = ops.coord[y];
    }
    r.set(xl,yl,xh,yh);
    return GOk;
}

Result  
GPolygon::addRing(int n_vertices, Coord* coords)
{
    return ops.addPart(n_vertices, coords);    
}

int
GPolygon::getRing(int nth, Coord*& coords)
{
    int num = ops.getPart(nth, coords);
    return num;
}


int     
GPolygon::bearing  (const GADT& other) const
{
    if(other.type()!=_Polygon) msgWarn("incompatible type");
    return 0;
    
}

double  
GPolygon::distance (const GADT& other) const
{
    GPoint this_cp, other_cp;
    if(other.type() != _Polygon) {
	msgWarn("GPolygon::distance: type mismatch");
	return 0;
    }
    
    centroid(this_cp);
    ((GPolygon&)other).centroid(other_cp);

    return this_cp.distance(other_cp);
}


//
// calls _area() and return a positive area.
//
double  
GPolygon::area () const
{
    double a = sarea();
    if(a>0) return a;
    else  return (a<0) ? (0-a) : a;
}


double
GPolygon::sarea() const
{
    if(ops.n_parts<=0 || ops.n_points<=0) return 0;
    
    double _area=0, *coo;
    int count=0;
    for(int i=0; i<ops.n_parts; i++) {
	coo = & ops.coord[count*PolyDim];
	_area += sarea(ops.parts[i], coo);
	count += ops.parts[i];
    }
    return _area;
}

double
GPolygon::sarea     (int n_point, Coord* coord) const
{
    register int i,j;
    double p, a_tmp = 0;
    if(n_point<3) return 0;
    for(i = n_point-1, j=0; j<n_point; i=j, j++) {
	p  = coord[i*2]*coord[j*2+1] -
	     coord[j*2]*coord[i*2+1];
	a_tmp += p;
    }
    return a_tmp/2;
}
    
double 
GPolygon::perimeter () const
{
    return ops.totalLength();
}

double
GPolygon::perimeter(int nth) const
{
    return ops.partLength(nth);
}
    
double
GPolygon::centroid  (GPoint& ctr) const
{
    return centroid(ops.parts[0], ops.coord, ctr);
}

double
GPolygon::centroid (int n_point, Coord* coord, GPoint& ctr) const
{
    register  int i,j;
    double p, a_tmp = 0, x_tmp=0, y_tmp=0;

    ctr.setX(InvalidCoord);
    ctr.setY(InvalidCoord);
    
    if(n_point<3) return 0;
    for(i = n_point-1, j=0; j<n_point; i=j, j++) {
	//x[i]*y[j] - y[i]*x[j]
	p  = coord[i*2]*coord[j*2+1] - coord[j*2]*coord[i*2+1];
	a_tmp += p;
	x_tmp += (coord[j*2] + coord[i*2])*p;
	y_tmp += (coord[j*2+1] + coord[i*2+1])*p;
    }
    if(a_tmp!=0) {
	ctr.setX( (Coord)(x_tmp / (3*a_tmp)) );
	ctr.setY( (Coord)(y_tmp / (3*a_tmp)) );
    }

    return (a_tmp/2); //return the signed area.
}
    

// Given a directed line pq, determine
// whether qr turns CW or CCW.	       
int 
GPolygon::whichSide(GPoint& p, GPoint& q, GPoint& r) 
{
    double result;
    result = (p.x - q.x) * (q.y - r.y) - (p.y - q.y) * (q.x - r.x);
    if (result < 0) return -1;	// q lies to the left  (qr turns CW).	
    if (result > 0) return  1;	// q lies to the right (qr turns CCW).
    return 0;			// q lies on the line from p to r.
}


//Lexicographic comparison of p and q
int 
GPolygon::pCompare(GPoint& p, GPoint& q) const
{
    if (p.x < q.x) return -1;	// p is less than q.
    if (p.x > q.x) return  1;	// p is greater than q.
    if (p.y < q.y) return -1;	// p is less than q.
    if (p.y > q.y) return  1;	// p is greater than q.
    return 0;			// p is equal to q.
}


// get next vertices of the ring
// and return true, iff successful.
int 
GPolygon::getPoint(int total, int& cur, Coord* coo, GPoint& p) const
{
    if(cur<total) {
	p.set(coo[cur*PolyDim], coo[cur*PolyDim+1]);
	cur++;
	return 1;
    }
    else return 0;
}


// Read next point into 'next' until it
// is different from 'previous' and
// return true iff successful.	
int 
GPolygon::getDifferentPoint(int total, int& cur, Coord* coo,
			 GPoint& previous, GPoint& next) const
{
    while(cur<total){
	next.set(coo[cur*PolyDim], coo[cur*PolyDim+1]);
	cur++;
	if(pCompare(previous, next) != 0) return 1;
    }

    return 0;
}


int 
GPolygon::classify(int n_point, Coord* coord)
{
    if(n_point<=0 || !coord) return GCONVEX_DEGENERATE;

    register int 
	cur_dir, this_dir, this_sign, angle_sign = 0, dir_changes = 0;
    GPoint	 first, second, third, save_first, save_second;
    int  I=0;
    
    if ( !getPoint(n_point, I, coord, first) || 
	 !getDifferentPoint(n_point, I, coord, first, second) )
	return GCONVEX_DEGENERATE;

    save_first = first;	save_second = second;
    cur_dir = pCompare(first, second);
    while( getDifferentPoint(n_point, I, coord, second, third) ) {
	CheckTriple;
    }
    /* Must check that end of list continues back to start properly */
    if ( pCompare(second, save_first) ) {
	third = save_first; CheckTriple;
    }
    third = save_second;  CheckTriple;

    if ( dir_changes > 2 ) return angle_sign ? GNOT_CONVEX : GNOT_CONVEX_DEGENERATE;
    if ( angle_sign  > 0 ) return GCONVEX_CCW;
    if ( angle_sign  < 0 ) return GCONVEX_CW;
    return GCONVEX_DEGENERATE;
}
    
int
GPolygon::classify()
{
    return classify(ops.parts[0], ops.coord);
}


int     
GPolygon::isConvex  () 
{
    switch (classify()) {
    case GCONVEX_CW:
    case GCONVEX_CCW: return true;
    default:	return false;
    }
}

int 
GPolygon::covers (const GSpatial& a) const
{
    GType t = a.type();
    if(t==_Point) return _covers( (const GPoint&)(a) );
    else if(t==_Segment) return _covers( (const GSegment&)(a));
    else if(t==_Arc) return _covers((const GArc&)(a));
    else if(t==_Polygon) return _covers((const GPolygon&)(a));
    else return UNDEFINED;
}

int    
GPolygon::isInside     (const GRect& rect) const
{
    GRect mbr(ops.xl, ops.yl, ops.xh, ops.yh);
    if(rect.covers(mbr)) return 1;
    
    GPoint pnt;
    for(int i=0; i<ops.parts[0]; i++) {
	pnt.set(ops.coord[i*PolyDim], ops.coord[i*PolyDim+1]);
	if(!pnt.isInside(rect)) return 0;
    }
    return 1;
}


//
// check if this polygon falls inside a circle,
// simply loop over every vetice.
//
int    
GPolygon::isInside     (const GCircle& c) const
{
    GRect mbr(ops.xl, ops.yl, ops.xh, ops.yh);
    if(c.covers(mbr)) return 1;

    GPoint pnt;
    for(int i=0; i<ops.parts[0]; i++) {
	pnt.set(ops.coord[i*PolyDim], ops.coord[i*PolyDim+1]);
	if(!pnt.isInside(c)) return 0;
    }
    return 1;
}


int    
GPolygon::_covers (const GPoint& p) const
{
    register  int i,j,c=0;
    Coord x=p.x, y=p.y;
    
    for(i=0, j=ops.parts[0]-1; i < ops.parts[0]; j = i++) {
	if((( (ops.coord[i*2+1] <= y) && (y<ops.coord[j*2+1])) ||
	    ( (ops.coord[j*2+1] <= y) && (y<ops.coord[i*2+1])) ) &&
	   (x < (ops.coord[j*2] - ops.coord[i*2]) * (y-ops.coord[i*2+1])/
	   (ops.coord[j*2+1]-ops.coord[i*2+1]) + ops.coord[i*2]))
	    
	    c=!c;
    }
    return c;
}


int    
GPolygon::_covers (const GSegment& s) const
{
    GPoint p;
    
    p.set(s.getX1(), s.getY1());
    if(! _covers(p)) return 0;
    p.set(s.getX2(), s.getY2());
    if(! _covers(p)) return 0;
    
    //now comes the the hard part... 
    return UNDEFINED;
}


int    
GPolygon::_covers (const GArc& s) const
{
    GRect mbr1, mbr2;
    
    s.mbr(mbr1);
    this->mbr(mbr2);
    
    if(! mbr2.covers(mbr1)) return 0;
    
    return UNDEFINED;
}


int    
GPolygon::_covers (const GPolygon& a) const
{
    GRect mbr1, mbr2;
    
    a.mbr(mbr1);
    this->mbr(mbr2);
    if(! mbr2.covers(mbr1)) return 0;
    
    //now comes the hard part
    return UNDEFINED;
}


int
GPolygon::intersects(int n_point, Coord* coo) const 
{
    return ops.intersects(n_point, coo);
}

int
GPolygon::intersects(const GOps& o) const
{
    return ops.intersects(o);
}

int 
GPolygon::intersects (const GSpatial& a) const
{
    GDBG("polygon intersecting %s ", GADTManager::getName(a.type()) );
    
    switch(a.type()) {
    case _Segment:	return _intersects( (const GSegment&)(a) );
	break;
    case _Rect:		return _intersects( (const GRect&) (a) );
	break;
    case _Circle:	return _intersects( (const GCircle&) a );
	break;
    case _Arc:		return _intersects( (const GArc&) (a) );
	break;
    case _Polygon:	return _intersects( (const GPolygon&) (a) );
	break;
	
    default:		msgFail("Polygon::intersects : Invalid type of operand!");
	break;
    }
}


//
// check if the polygon intersects the segment.
//
int    
GPolygon::_intersects (const GSegment& seg_in) const
{
    return ops.intersects(seg_in);
}

//
// check if the polygon intersects with a polyline: O(n*n)
//
int    
GPolygon::_intersects (const GArc& other) const
{
    return other.intersects(ops);
}

	    
int    
GPolygon::_intersects (const GPolygon& other)const 
{
    return ops.intersects(other.ops);
}

//
// check if the polygon intersects a rectangle
//
int    
GPolygon::_intersects (const GRect&  r) const
{
    return ops.intersects(r);
}


//
// check if the polygon intesects with a circle.
//
int    
GPolygon::_intersects (const GCircle& c)const
{
    return ops.intersects(c);
}
    
ostream& operator << (ostream& s, const GPolygon& p)
{
    GPoint pt;
    
    s<<"(n_point="<<p.ops.n_points<< ", n_rings="<<p.ops.n_parts;
    s<<", PSize="<<p.getPSize()<<" area="<<p.area()<<", peri="<<p.perimeter();
    return s<<")"<<endl;
}


/*************************************************************************/
/******************************* THE  END ********************************/
/*************************************************************************/



