
/****************************************************************************
 *   st_rectADT.cc : 
 *   		implementation of rectangle ADT.
 *
 *   Change Log :
 *		 Dec-22-1996 Initial version,  Liujian Qian
 *
 *   $Id: gadt_rect.cpp,v 1.9 1998/12/10 00:11:31 qian Exp $
 ***************************************************************************/
#include <iostream.h>
#if defined(_WIN32)
#include <strstrea.h>
#else
#include <strstream.h>
#endif

#include "gfc.h"

int _GIRect::cnt;

 
GRect::GRect() 
{
    xlow = ylow = xhi = yhi = InvalidCoord;
}	

GRect::GRect(Coord xLow, Coord yLow, Coord xUp, Coord yUp)
{
    set(xLow, yLow, xUp, yUp);
}

//copy constructor
GRect::GRect(const GRect& other)
{ 
    xlow = other.xlow, ylow = other.ylow, xhi = other.xhi, yhi = other.yhi;
}

GRect&
GRect:: operator= (const GRect& other) 
{
    GADT::operator=(other);
    xlow = other.xlow, ylow = other.ylow, xhi = other.xhi, yhi = other.yhi;
    return *this;
}

uint32  	 
GRect::getPSize() const 
{
    return GADT::getPSize() + 4*sizeof(Coord);
}


//
// append persistent data into the pipe
//
Result 	
GRect::pack(DataPipe& p)
{
    GADT::pack(p);
    Coord tmp;
    if(GEndian::big) {
	tmp = xlow;
	flipDouble(&tmp);
	p.append(CONPTR(&tmp), sizeof(Coord));
	tmp = ylow;
	flipDouble(&tmp);
	p.append(CONPTR(&tmp), sizeof(Coord));
	tmp = xhi;
	flipDouble(&tmp);
	p.append(CONPTR(&tmp), sizeof(Coord));
	tmp = yhi;
	flipDouble(&tmp);
	p.append(CONPTR(&tmp), sizeof(Coord));
    } else {
	p.append( CONPTR(&xlow), sizeof(Coord));
	p.append( CONPTR(&ylow), sizeof(Coord));
	p.append( CONPTR(&xhi), sizeof(Coord));
	p.append( CONPTR(&yhi), sizeof(Coord));
    }
    
    return GOk;
}

//
// unpack data from the pipe
//
Result 
GRect::unpack(DataPipe& p)
{
    GADT::unpack(p);
    GAssert(!p.eop());
    p.fetch((char*)&xlow, sizeof(Coord));
    p.fetch((char*)&ylow, sizeof(Coord));
    p.fetch((char*)&xhi,  sizeof(Coord));
    p.fetch((char*)&yhi,  sizeof(Coord));

    if(GEndian::big) {
	flipDouble(&xlow);
	flipDouble(&ylow);
	flipDouble(&xhi);
	flipDouble(&yhi);	
    }
    
    return GOk;
}


//
// user format: "xlow ylow  xhi yhi"
//
Result 	 
GRect::input   (const char* in)
{
    GAssert(in);
    istrstream is(in, strlen(in));
    
    Coord x1,y1, x2,y2;

    is>>x1>>y1>>x2>>y2;

    set(x1,y1,x2,y2);	//this assures proper order of the two corners.
    
    return GOk;
}


Result 	 
GRect::output   (char*& out)
{	
    bp.recycle();
    char* p = (char*) bp.ptr();
    sprintf(p, " %f %f %f %f ", xlow, ylow, xhi, yhi);
    out = p;
    
    return GOk;
}
    


double
GRect::distance (const GRect& a) const
{
    return hypot( ((xlow+xhi)/2 - (a.xlow+a.xhi)/2), 
		  ((ylow+yhi)/2 - (a.ylow+a.yhi)/2) );
}

Result
GRect::scale      (double xf, double yf, double )
{
    xlow *= xf, xhi *= xf;
    ylow *= yf, yhi *= yf;
    return GOk;
}


Result
GRect::translate      (double xf, double yf, double )
{
    xlow += xf, xhi += xf;
    ylow += yf, yhi += yf;
    return GOk;
}


Result
GRect::rotate	   (double, bool)
{
    return GOk;
}

Result
GRect::transform  (const GMatrix&)
{
    return GOk;
}


//
// comparison is only allowed between same type of objects.
//
int		 
GRect::cmp (const GADT& other) const
{
    MATCHTYPE(other);
    
    if( equals( (const GRect&)other) ) return 0;
    else if(covers( (const GRect&)other) ) return 1;
    else if(((GRect&)other).covers(*this) ) return -1;
    else return UNDEFINED;
}


void 
GRect::centroid(Coord& x, Coord& y) 
{
    x = (xlow + xhi) / 2;
    y = (ylow + yhi) / 2;
}	

bool 
GRect::equals (const GRect& a) const
{
    return (xlow == a.xlow && ylow == a.ylow &&
	    xhi  == a.xhi  && yhi  == a.yhi);
}

void
GRect::enlarge (const GRect& other)
{
    if(xlow>other.xlow) xlow = other.xlow;
    if(ylow>other.ylow) ylow = other.ylow;
    if(xhi <other.xhi ) xhi  = other.xhi;
    if(yhi <other.yhi ) yhi  = other.yhi;
}

bool
GRect::isValid() const 
{
    if(xlow==InvalidCoord || ylow==InvalidCoord ||
       xhi ==InvalidCoord || yhi ==InvalidCoord) return false;
    if(xlow<xhi && ylow<yhi) return false;
    return true;
}

int  
GRect::covers    (const GSpatial& a) const
{
    switch(a.type()) {
    case _Point:	return _covers( (const GPoint&) (a) );
	break;
    case _Segment:	return _covers( (const GSegment&)(a) );
	break;
    case _Rect:		return _covers( (const GRect&) (a) );
	break;
    case _Arc:		return _covers( (const GArc&) (a) );
	break;
    case _Polygon:	return _covers( (const GPolygon&) (a) );
	break;
	
    default:		msgWarn("Invalid type of operand!");
	break;
    }
    return UNDEFINED;
}

//check if this rectangle covers a given rectangle
int
GRect::_covers(const GRect& a) const
{
    GDBG("rect covering %s ", GADTManager::getName(a.type()) );

    return (xlow <= a.xlow && xhi >= a.xhi &&
	    ylow <= a.ylow && yhi >= a.ylow );
}


//check if this rectangle covers a given segment
int
GRect::_covers(const GSegment& seg) const
{
    GDBG("rect covering %s ", GADTManager::getName(seg.type()) );

    GRect a;
    ((GSegment&)seg).mbr(a);

    return (xlow <= a.xlow && xhi >= a.xhi &&
	    ylow <= a.ylow && yhi >= a.ylow );
}


//check if this rectangle covers a point
int   
GRect::_covers(const GPoint& p) const
{
    GDBG("rect covering %s", GADTManager::getName(p.type()) );

    return p.isInside(*this);
}


//check if this rectangle covers a polyline
int   
GRect::_covers(const GArc& arc) const
{
    GDBG("rect covering %s ", GADTManager::getName(arc.type()) );

    return arc.isInside(*this);
}

//check if this rectangle covers a polygon
int
GRect::_covers(const GPolygon& pgn) const
{
    return pgn.isInside(*this);
}

int  
GRect::intersects(const GSpatial& a) const
{
    switch(a.type()) {
    case _Segment:	return _intersects( (const GSegment&)(a) );
	break;
    case _Rect:		return _intersects( (const GRect&) (a) );
	break;
    case _Circle:	return _intersects( (const GCircle&) a );
	break;
    case _Arc:		return _intersects( (const GArc&) (a) );
	break;
    case _Polygon:	return _intersects( (const GPolygon&) (a) );
	break;
	
    default:		msgWarn("Invalid type of operand!");
	break;
    }
    return UNDEFINED;
}


//check if this rectangle intersects another
int 
GRect::_intersects(const GRect& a) const
{
    GDBG("rect intersecting %s ", GADTManager::getName(a.type()) );

    //if one rectangle is inside of another
    //if(_covers(a)) return true;
    //else if(a.covers(*this)) return true;
    //else
    return (!(xlow > a.xhi || a.xlow > xhi ||
	      ylow > a.yhi || a.ylow > yhi));
}	

//
//check if this rectangle intersects a circle 
//call circle's intersects(rect) method.
//
int
GRect::_intersects(const GCircle& a) const
{
    GDBG("rect intersecting %s", GADTManager::getName(a.type()) );

    return a.intersects(*this) ;
}	

int   
GRect::_intersects(const GArc& ln) const
{
    GDBG("rect intersecting %s ", GADTManager::getName(ln.type()) );

    return ln.intersects(*this);
}

int	  
GRect::_intersects(const GSegment& seg) const
{
    GDBG ("rect intersecting %s ", GADTManager::getName(seg.type()) );

    return seg.intersects(*this);
}

int  
GRect:: _intersects(const GPolygon& pgn) const
{
    GDBG("rect intersecting %s ", "Polygon" );

    return pgn.intersects(*this);
}

ostream&  operator<<(ostream& s, const GRect& r)
{
    return s<<"("<<r.xlow<<", "<<r.ylow<<", "<<r.xhi<<", "<<r.yhi<<")";
}


