/****************************************************************************
 *   gtime_adt.cc : 
 *   		    Implementation of timestamp and related stuff.
 *
 *   Change Log :
 *		 09-30-1996 Initial version,  Liujian Qian
 *
 *   $Id: gadt_time.cpp,v 1.5 1998/12/10 00:11:34 qian Exp $
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#if defined(__WIN32__) || defined(_WIN32)
#include <strstrea.h>
#else
#include <strstream.h>
#endif

#include "gdatapipe.h"
#include "gadt_time.h"

int _GITimestamp::cnt;

//these are  two special timestamps declared globally.
GTimestamp ORIGIN("ORIGIN");
GTimestamp FOREVER("FOREVER");
char* GDate::month_names[] = {"BAD", "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
			 "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
uint8 GDate::days_in_mon[] = {0, 31, 28, 31, 30, 31, 30, 
			      31, 31, 30, 31, 30, 31};

Months GDate::nameToMonth(const char* mon)
{
    char umon[24], *p=umon;
    while(*mon) *p++ = (char)toupper(*mon++);
    umon[3]=0;
    for(int i=1; i<13; i++) 
	if(!strcmp(month_names[i], umon)) return (Months)i;
    return _BAD;
}

    
///
//
long    
GTimestamp::_seconds[6] = {   
                         1,		//seconds/second
			 60,		//seconds/minute
			 3600,  	//seconds/hour
			 86400, 	//seconds/day
			 86400*7,	//seconds/week
			 86400*356	//seconds/year (non-leap year)
};

/*			 
 *-------------------------------------------------------------------------
 * GTimestamp  --
 *		various types of constructors.
 *
 * Results:
 * 	Taking hints and make new timestamp usable.
 *-------------------------------------------------------------------------
 */
GTimestamp::GTimestamp ()
{
    init();
}

GTimestamp::GTimestamp (char* special)
{
    if(strcmp(special, "ORIGIN")==0)
	setToOrigin();
    else if (strcmp(special, "FOREVER")==0)
	setToForever();
    else {
	t.ts.type = 4;		//regular timestamp.
	t.ts.sign = 0;		//before origin.
	t.ts.secs_hi = 0;	//clean all bits
	t.ts.secs_lo = 0;	//for seconds since origin.
    }
}

GTimestamp::GTimestamp(const GDate& d) 
{	
    date2TS(&d);
}

GTimestamp::GTimestamp(int year)
{
    GDate d(year, 0, 0, 0, 0, 0);
    date2TS(&d);
}

GTimestamp::GTimestamp(int year, int mon, int day)
{
    GDate d(year, mon, day, 0, 0, 0);
    date2TS(&d);
}

GTimestamp::GTimestamp(int year, int mon, int day, int hour, int minute, int sec)
{
    GDate d(year, mon, day, hour, minute, sec);
    date2TS(&d);
}

GTimestamp::GTimestamp(const GTimestamp& copy)
{
    t.val = copy.t.val;
}

GTimestamp&
GTimestamp::operator= (const GTimestamp& other)
{
    t.val = other.t.val;
    return *this;
}

Result
GTimestamp::init      (void) 
{
    t.ts.type = 4;	//regular timestamp.
    t.ts.sign = 0;	//before origin.
    t.ts.secs_hi = 0;	//clean all bits
    t.ts.secs_lo = 0;	//for seconds since origin.
    return GOk;
}
    

uint32	 
GTimestamp::getPSize() const
{
    return GADT::getPSize() + sizeof(int64);
}


Result 	 
GTimestamp::pack  (DataPipe& p)
{
    GADT::pack(p);

    p.append(CONPTR(&t.val), sizeof(int64));

    return GOk;
}

Result 	 
GTimestamp::unpack(DataPipe& p)
{
    GADT::unpack(p);    

    GAssert(!p.eop());
    p.fetch((char*)&(t.val),  sizeof(int64));
    
    return GOk;
}

//
// 
//
Result 	
GTimestamp::input  (const char* in)
{
    GAssert(in);
    
    GDate d;
    char mon[4];
    int date,h,m,s,y;
    sscanf(in, "%s %2d %d:%d:%d %d", mon, &date, &h,&m, &s, &y);
    mon[3]=0;
    d.month = d.nameToMonth(mon);
    d.day = (short)date, d.hour=(short)h, d.minute=(short)m;
    d.second=(short)s,d.year=(int64)y;
    date2TS(&d);

    return GOk;
}


Result 	 
GTimestamp::output  (char*& out)
{
    GDate d;
    
    if(t.ts.type!=TS_REGULAR) {
	msgWarn("Not regular timestamp!");
	out = 0;
	return GFail;
    }

    bp.extendTo(128);
    bp.recycle();
    char* asc= (char*) bp.ptr();
    
    date(&d);
    sprintf(asc, "%s %2d %d:%d:%d GMT %d", 
	    GDate::month_names[d.month], d.day, d.hour, 
	    d.minute, d.second, (int)(d.year));
    out = asc;
    return GOk;
}


Result 	 
GTimestamp::output  (char*& out, TimeUnit u )
{
    GDate d;

    if(t.ts.type!=TS_REGULAR) {
	msgWarn("Not regular timestamp!");
	out = 0;
	return GFail;
    }
    bp.extendTo(128);
    bp.recycle();
    char* asc= (char*)bp.ptr();
    
    date(&d);
    if(u==_SECOND)
	sprintf(asc, "%s %2d %d:%d:%d GMT %d", 
		GDate::month_names[d.month], d.day, d.hour, 
		d.minute, d.second, (int)(d.year));
    else if(u==_MINUTE)
	sprintf(asc, "%s %2d %d:%d GMT %d", 
		GDate::month_names[d.month], d.day, d.hour,
		d.minute, (int)(d.year));
    else if(u==_HOUR)
	sprintf(asc, "%s %2d %d GMT %d", 
		GDate::month_names[d.month], d.day, d.hour, (int)(d.year));
    else if(u==_DAY)
	sprintf(asc, "%s %2d %d", 
		GDate::month_names[d.month], d.day, (int)(d.year));
    else if(u==_YEAR)
	sprintf(asc, "%d", (int)(d.year));
    else if(u==_MONTH)
	sprintf(asc, "%s %d", GDate::month_names[d.month], (int)(d.year));
    else 
	sprintf(asc, "bad precision");
    
    out = asc;
    
    return GOk;
}

int
GTimestamp::cmp(const GADT& a) const
{
    MATCHTYPE(a);
    if(*this > (GTimestamp&)a) return 1;
    else if(*this < (GTimestamp&)a) return -1;
    else return 0;
}
    
Result
GTimestamp::scale  (int) 
{
    return GOk;		//not defined
}


int
GTimestamp::before	  (const GTemporal& a) const
{
    return *this < (GTimestamp&)a;
}

int
GTimestamp::after	  (const GTemporal& a) const
{
    return *this > (GTimestamp&)a;
}

int
GTimestamp::during	  (const GTemporal& ) const
{
    return 0;		//not defined
}

int
GTimestamp::overlap	  (const GTemporal& ) const
{
    return 0;		//not defined
}


ostream& operator << (ostream& s, GTimestamp& t)
{
    char* p=0;
    t.output(p);
    if(p) s<<p;
    return s;
}

bool 
GTimestamp::operator!=(GTimestamp st2) const
{
    return (t.val != st2.t.val);
}

bool 
GTimestamp::operator==(GTimestamp st2) const
{
    return (t.val == st2.t.val);
}

bool 
GTimestamp::operator< (GTimestamp st2)  const
{
    if( st2.t.ts.type==TS_REGULAR && 
	t.ts.type==TS_REGULAR ) {
	if(t.ts.sign == 0 && st2.t.ts.sign == 0) 
	    //!!! because the father away backward (geater val), 
	    //!!! the smaller(earlier) the time
	    return (t.val > st2.t.val); 
	else	
	    return (t.val < st2.t.val);
    }
    else if(st2.t.ts.type == TS_SPECIAL && 
	    t.ts.type == TS_REGULAR){
	if(st2.t.ts.secs_lo == 1) return FALSE;
	else if (st2.t.ts.secs_lo == 2) return TRUE;
	else return FALSE;	//what's this?
	}
    else  if(st2.t.ts.type == TS_REGULAR &&
	     t.ts.type == TS_SPECIAL) {
	if(t.ts.secs_lo == 1) return TRUE;
	else if(t.ts.secs_lo ==2) return FALSE;
	else return FALSE;
    }
}

bool 
GTimestamp::operator<= (GTimestamp st2) const
{
    return ( t.val==st2.t.val || (*this<st2) );
}

bool 
GTimestamp::operator>=(GTimestamp st2) const
{
    return (! (*this<st2) );
}

bool 
GTimestamp::operator> (GTimestamp st2) const
{
    return (t.val!=st2.t.val && !(*this<st2));
}


/*
 *-------------------------------------------------------------------------
 * GTimestamp::operator+=  --
 *	This method and the rest in the group override arithmetic
 *	operators.
 * Results:
 *	N/A
 * Side effects:
 * 	N/A
 *-------------------------------------------------------------------------
 */
GTimestamp&
GTimestamp::operator+=(uint64 seconds)
{
    uint64 tmp_seconds=0;

    if( !(t.ts.type==TS_REGULAR || t.ts.type==TS_SPAN) ){
	msgWarn("Invalid timestamp type in operator +=");
	return *this;
    }
    
    tmp_seconds = t.ts.secs_hi;
    tmp_seconds <<= 32ll;
    tmp_seconds += t.ts.secs_lo;

    if(t.ts.sign == 0) {
	if (tmp_seconds <= seconds) {
	    t.ts.sign = 1;
	    tmp_seconds = seconds-tmp_seconds;
	} else {
	    tmp_seconds -= seconds;
	}
    } else {  //we are  after origin; simply add the seconds
	tmp_seconds += seconds;
    }

    if(tmp_seconds <= 0xFFFFFFFFull) {
	t.ts.secs_hi = 0ll;
	t.ts.secs_lo = tmp_seconds;
    } else {
	t.ts.secs_lo = tmp_seconds & 0x0FFFFFFFFull;
	t.ts.secs_hi = (tmp_seconds & 0x07FFFFFF00000000ull) >> 32ll;
    }    

    return *this;
}


GTimestamp&
GTimestamp::operator-=(uint64 seconds)
{
    uint64 tmp_seconds=0;

    if( !(t.ts.type==TS_REGULAR || t.ts.type==TS_SPAN) ){
	msgWarn("Invalid timestamp type in operator -=");
	return *this;
    }
    
    tmp_seconds = t.ts.secs_hi;
    tmp_seconds <<= 32ll;
    tmp_seconds += t.ts.secs_lo;

    if(t.ts.sign == 1) {
	if (tmp_seconds <= seconds) {
	    t.ts.sign = 0;
	    tmp_seconds = seconds-tmp_seconds;
	} else {
	    tmp_seconds -= seconds;
	}
    } else {  //we are  before origin; simply ADD the seconds
	tmp_seconds += seconds;
    }

    if(tmp_seconds <= 0xFFFFFFFFull) {
	t.ts.secs_hi = 0ll;
	t.ts.secs_lo = tmp_seconds;
    } else {
	t.ts.secs_lo = tmp_seconds & 0x0FFFFFFFFull;
	t.ts.secs_hi = (tmp_seconds & 0x07FFFFFF00000000ull) >> 32ll;
    }    

    return *this;
}

// set timestamp to just indicate the year.
void 
GTimestamp::setYear(int year)
{
    GDate d(year, 0,0, 0, 0, 0);
    date2TS(&d);
}
    

/*
 *-------------------------------------------------------------------------
 * GTimestamp::setToOrigin --
 *	This method turns the timestamp into a special type one
 *	representing beginning of the universe (big bang). This
 *	timestamp is smaller than any other timestamp.          
 * Results:
 *	N/A
 * Side effects:
 * 	N/A
 *-------------------------------------------------------------------------
 */
void
GTimestamp::setToOrigin(void)
{
    t.ts.type = TS_SPECIAL;
    t.ts.sign = 0;
    t.ts.secs_hi = 0;
    t.ts.secs_lo = 1;
}

/*
 *-------------------------------------------------------------------------
 * GTimestamp::setToForever --
 *	This turns the timestamp into a "forever" one,
 *	which is greater than any other timestamp.    
 * Results:
 *	N/A
 * Side effects:
 * 	N/A
 *-------------------------------------------------------------------------
 */
void
GTimestamp::setToForever(void)
{
    t.ts.type = TS_SPECIAL;
    t.ts.sign = 1;
    t.ts.secs_hi = 0;
    t.ts.secs_lo = 2;
}


/*
 *-------------------------------------------------------------------------
 * GTimestamp::setToSpan --
 *	This method turn the timestamp into a fixed span.
 * Results:
 *	The seconds become (is set to) the duration; sign has no meaning.
 * Side effects:
 * 	N/A
 *-------------------------------------------------------------------------
 */
void
GTimestamp::setToSpan(int64 seconds	//IN, default to -1
			)
{
    t.ts.type = TS_SPAN;
    if(seconds!=-1){	//we will use the new value for the span
	if((uint64)seconds <= 0xFFFFFFFFull) {
	    t.ts.secs_hi = 0ll;
	    t.ts.secs_lo = seconds;
	} else {
	    t.ts.secs_lo = seconds & 0x0FFFFFFFFull;
	    t.ts.secs_hi = (seconds & 0x07FFFFFF00000000ull) >> 32ll;
	}
    }
}



/*
 *-------------------------------------------------------------------------
 * GTimestamp :: timeLabel --
 *	Return a time label that is in user format and in accordance to
 *	the precision. (For example, if the precision is set to year, the
 *	time label will only includ the year number, such as "1996", instead
 *	of "JAN 1 00:00:00 1996".)                                          
 *
 * Results:
 *	GOk upon success. Otherwise GFail. The actual label is returned
 *	through 2nd argument; it must have enough space preallocated.
 *
 * Side effects:
 * 	N/A
 *-------------------------------------------------------------------------
 */
Result
GTimestamp::timeLabel(TimeUnit precision, char* s)
{
    GDate d;

    if(!s) msgFail("invalid pointer for holding time label!");
    
    //first convert the timestamp into standard ascii format.
    date(&d);
    char str[128], *tmp;
    
    //extract propriate field from the ascii string according to the
    //precision of this timestamp.
    switch( precision ){
    case _SECOND:	output(tmp);
	                strcpy(s, tmp); 
			return GOk;
    case _MINUTE:	sprintf(str, "%s %2d %d:%d:00 %d", 
				GDate::month_names[d.month], d.day,
				d.hour, d.minute, (int)d.year);
                        break;
    case _HOUR:		sprintf(str, "%s %2d %d:00 %d", 
				GDate::month_names[d.month], d.day, d.hour, 
				(int)d.year);
                        break;
    case _DAY:		sprintf(str, "%s %2d %d:00 %d", 
				GDate::month_names[d.month], d.day, d.hour, 
				(int)d.year);
    case _WEEK:		break;
    case _MONTH:	sprintf(str, "%s %d", 
				GDate::month_names[d.month], (int)d.year);
	                break;
    case _YEAR:	        sprintf(str, "%d", (int)d.year);
	                break;
    default:            sprintf(str, "This precision not supported!");
	                return GFail;
	                break;
    }

    strcpy(s, str);

    return GOk;
}

	
/*
 *-------------------------------------------------------------------------
 * GTimestamp :: date2TS  --
 *
 *	convert a gregorian date into timestamp (seconds since origin.)
 *	Originally written by "MultiCal" group; modified to support hour/min/sec.
 * Results:
 *	The sign bit and timestamp type are set properly. 
 * Side effects:
 *-------------------------------------------------------------------------
 */
Result
GTimestamp::date2TS (const GDate* greg)
{
    int64 seconds, true_year;
    long years;
    long probe;
    register unsigned long i, templong;

    true_year = greg->year-1; // Adjust for count starting at 1AD 
    seconds = 0;
    years = 0;

    // Figure out the number of seconds corresponding to the years
    // Do high order word
    // To avoid a 64-bit divide, we are gonna do repeated subtraction
    // To avoid a 64-bit shifts, we are gonna do one word of seconds 
    // at a time 
    templong = true_year;

    // unroll loop
    if ((i = templong & 0x0000000F)) {
        seconds += years_to_gregs_nibbles[0][i];
        years += years_to_gregs_residue_nibbles[0][i];
    }
    if ((i = templong & 0x000000F0)) {
	i >>= 4;
        seconds += years_to_gregs_nibbles[1][i];
        years += years_to_gregs_residue_nibbles[1][i];
    }
    if ((i = templong & 0x00000F00)) {
	i >>= 8;
        seconds += years_to_gregs_nibbles[2][i];
        years += years_to_gregs_residue_nibbles[2][i];
    }
    if ((i = templong & 0x0000F000)) {
	i >>= 12;
        seconds += years_to_gregs_nibbles[3][i];
        years += years_to_gregs_residue_nibbles[3][i];
    }
    if ((i = templong & 0x000F0000)) {
	i >>= 16;
        seconds += years_to_gregs_nibbles[4][i];
        years += years_to_gregs_residue_nibbles[4][i];
    }
    if ((i = templong & 0x00F00000)) {
	i >>= 20;
        seconds += years_to_gregs_nibbles[5][i];
        years += years_to_gregs_residue_nibbles[5][i];
    }
    if ((i = templong & 0x0F000000)) {
	i >>= 24;
        seconds += years_to_gregs_nibbles[6][i];
        years += years_to_gregs_residue_nibbles[6][i];
    }
    if ((i = templong & 0xF0000000)) {
	i >>= 28;
        seconds += years_to_gregs_nibbles[7][i];
        years += years_to_gregs_residue_nibbles[7][i];
    }

    // Do first four bits of low order word
    if ( (i = true_year >> 32)) {
        seconds += years_to_gregs_nibbles[8][i];
        years += years_to_gregs_residue_nibbles[8][i];
    }

    // Reapportion years into gregs
  
    if (years >= 2800)
	if (years >= 4000)
	    if (years >= 4800)
		probe = (years >= 5200)? 13 : 12;
	    else
		probe = (years >= 4400)? 11 : 10;
	else
	    if (years >= 3200)
		probe = (years >= 3600)? 9 : 8;
	    else
		probe = 7;
    else
	if (years >= 1200)
	    if (years >= 2000)
		probe = (years >= 2400)? 6 : 5;
	    else
		probe = (years >= 1600)? 4 : 3;
	else
	    if (years >= 400)
		probe = (years >= 800)? 2 : 1;
	    else
		probe = 0;

    years -= yr[probe];
    seconds += sg[probe];
    seconds += year_to_seconds[years];

    // Figure out the number of seconds corresponding to the months
    seconds += (leap_year[years])? leap_month_to_seconds[greg->month] :
	                           month_to_seconds[greg->month];

    // Figure out the number of seconds corresponding to the days
    seconds += day_to_seconds[greg->day];

    // Figure out the number of seconds corresponding to hours, mins...
    seconds += greg->hour * 3600;
    seconds += greg->minute * 60;
    seconds += greg->second;
    
    seconds = seconds - SECONDS_TO_1970;
    // cout<<"remaining seconds=" << seconds;
     
    t.ts.type = TS_REGULAR;
    if (seconds <=0ll) {
	t.ts.sign = 0;
	seconds = 0ll-seconds;
	//printf("minus seconds= %lld, or %llx\n", seconds, seconds);
    } else 
	t.ts.sign = 1;
 
   if((uint64)seconds <= 0xFFFFFFFFull) {
	t.ts.secs_hi = 0ll;
	t.ts.secs_lo = seconds;
	//printf("in low\n");
    } else {
	t.ts.secs_lo = seconds & 0x0FFFFFFFFull;
	t.ts.secs_hi = (seconds & 0x07FFFFFF00000000ull) >> 32ll;
    }    

   return GOk;
}


/*
 *-------------------------------------------------------------------------
 * GTimestamp :: date  --
 *	Translate from seconds format to explicit Greogrian date.
 *	Original version by "MultiCal" group; modified to  support
 *	hour/min/sec.                                             
 * Results:
 *	Result is returned through the argument greg.
 * Side effects:
 * 	N/A
 *-------------------------------------------------------------------------
 */
Result
GTimestamp::date (GDate* greg) const
{
    int64 seconds, templl;
    long years, seconds_residue, gregs, days_residue;
    register unsigned long templong, i;
    int probe;
    
    if(t.ts.type!=TS_REGULAR) msgFail("Invalid timestamp!");

    seconds = t.ts.secs_hi;
    seconds <<= 32ll;
    seconds += t.ts.secs_lo;
    if(t.ts.sign==0)  //before origin
	seconds = 0ll - seconds;

    // Move origin to 1970 
    seconds +=  SECONDS_TO_1970;
    gregs = 0;
    days_residue = 0;
    seconds_residue = 0;

    // Figure out gregs;  1 greg is (86400 * (365*400 + 97)) 
    // To avoid a 64-bit divide, we are gonna do repeated subtraction 
    // To avoid a 64-bit shifts, we are gonna do one word of seconds 
    // at a time 

    // Do high order word 
    templong = seconds;

    // unroll loop
    if ((i = templong & 0x0000000F)) {
        gregs += greg_nibbles[0][i];
        seconds_residue += second_nibbles[0][i];
        days_residue += day_nibbles[0][i];
    }
    if ((i = templong & 0x000000F0)) {
	i >>= 4;
        gregs += greg_nibbles[1][i];
        seconds_residue += second_nibbles[1][i];
        days_residue += day_nibbles[1][i];
    }
    if ((i = templong & 0x00000F00)) {
	i >>= 8;
        gregs += greg_nibbles[2][i];
        seconds_residue += second_nibbles[2][i];
        days_residue += day_nibbles[2][i];
    }
    if ((i = templong & 0x0000F000)) {
	i >>= 12;
        gregs += greg_nibbles[3][i];
        seconds_residue += second_nibbles[3][i];
        days_residue += day_nibbles[3][i];
    }
    if ((i = templong & 0x000F0000)) {
	i >>= 16;
        gregs += greg_nibbles[4][i];
        seconds_residue += second_nibbles[4][i];
        days_residue += day_nibbles[4][i];
    }
    if ((i = templong & 0x00F00000)) {
	i >>= 20;
        gregs += greg_nibbles[5][i];
        seconds_residue += second_nibbles[5][i];
        days_residue += day_nibbles[5][i];
    }
    if ((i = templong & 0x0F000000)) {
	i >>= 24;
        gregs += greg_nibbles[6][i];
        seconds_residue += second_nibbles[6][i];
        days_residue += day_nibbles[6][i];
    }
    if ((i = templong & 0xF0000000)) {
	i >>= 28;
        gregs += greg_nibbles[7][i];
        seconds_residue += second_nibbles[7][i];
        days_residue += day_nibbles[7][i];
    }

    // Do low order word
    templong = seconds >> 32;

    // unroll loop
    if ((i = templong & 0x0000000F)) {
        gregs += greg_nibbles[8][i];
        seconds_residue += second_nibbles[8][i];
        days_residue += day_nibbles[8][i];
    }
    if ((i = templong & 0x000000F0)) {
	i >>= 4;
        gregs += greg_nibbles[9][i];
        seconds_residue += second_nibbles[9][i];
        days_residue += day_nibbles[9][i];
    }
    if ((i = templong & 0x00000F00)) {
	i >>= 8;
        gregs += greg_nibbles[10][i];
        seconds_residue += second_nibbles[10][i];
        days_residue += day_nibbles[10][i];
    }
    if ((i = templong & 0x0000F000)) {
	i >>= 12;
        gregs += greg_nibbles[11][i];
        seconds_residue += second_nibbles[11][i];
        days_residue += day_nibbles[11][i];
    }
    if ((i = templong & 0x000F0000)) {
	i >>= 16;
        gregs += greg_nibbles[12][i];
        seconds_residue += second_nibbles[12][i];
        days_residue += day_nibbles[12][i];
    }
    if ((i = templong & 0x00F00000)) {
	i >>= 20;
        gregs += greg_nibbles[13][i];
        seconds_residue += second_nibbles[13][i];
        days_residue += day_nibbles[13][i];
    }
    if ((i = templong & 0x0F000000)) {
	i >>= 24;
        gregs += greg_nibbles[14][i];
        seconds_residue += second_nibbles[14][i];
        days_residue += day_nibbles[14][i];
    }

    // Reapportion the leftover seconds into days using binary search 
    if (seconds_residue >= 864000)
	if (seconds_residue >= 1296000)
	    if (seconds_residue >= 1468800)
		if (seconds_residue >= 1555200)
		    probe = (seconds_residue >= 1641600)? 19 : 18;
		else
		    probe = 17;
	    else
		probe = (seconds_residue >= 1382400)? 16 : 15;
	else
	    if (seconds_residue >= 1036800)
		if (seconds_residue >= 1123200)
		    probe = (seconds_residue >= 1209600)? 14 : 13;
		else
		    probe = 12;
	    else
		probe = (seconds_residue >= 950400)? 11 : 10;
    else
	if (seconds_residue >= 432000)
	    if (seconds_residue >= 604800)
		if (seconds_residue >= 691200)
		    probe = (seconds_residue >= 777600)? 9 : 8;
		else
		    probe = 7;
	    else
		probe = (seconds_residue >= 518400)? 6 : 5;
	else
	    if (seconds_residue >= 172800)
		if (seconds_residue >= 259200)
		    probe = (seconds_residue >= 345600)? 4 : 3;
		else
		    probe = 2;
	    else
		probe = (seconds_residue >= 86400)? 1 : 0;

    days_residue += probe;
    seconds_residue -= sr[probe];

    // Reapportion the leftover days into gregs using binary search
    if (days_residue >= 1460970)
	if (days_residue >= 2191455)
	    if (days_residue >= 2483649)
		if (days_residue >= 2629746)
		    probe = (days_residue >= 2775843)? 19 : 18;
		else
		    probe = 17;
	    else
		probe = (days_residue >= 2337552)? 16 : 15;
	else
	    if (days_residue >= 1753164)
		if (days_residue >= 1899261)
		    probe = (days_residue >= 2045358)? 14 : 13;
		else
		    probe = 12;
	    else
		probe = (days_residue >= 1607067)? 11 : 10;
    else
	if (days_residue >= 730485)
	    if (days_residue >= 1022679)
		if (days_residue >= 1168776)
		    probe = (days_residue >= 1314873)? 9 : 8;
		else
		    probe = 7;
	    else
		probe = (days_residue >= 876582)? 6 : 5;
	else
	    if (days_residue >= 292194)
		if (days_residue >= 438291)
		    probe = (days_residue >= 584388)? 4 : 3;
		else
		    probe = 2;
	    else
		probe = (days_residue >= 146097)? 1 : 0;

    gregs += probe;
    days_residue -= dr[probe];

    // Figure out year and day in year within the last greg portion
    if (days_residue >= 73048)
	if (days_residue >= 109572)
	    if (days_residue >= 127834)
		if (days_residue >= 136965)
		    if (days_residue >= 141348)
			if (days_residue >= 143540)
			    if (days_residue >= 144636)
				if (days_residue >= 145366)
				    probe = (days_residue >= 145731)? 399 : 398;
				else
				    probe = (days_residue >= 145001)? 397 : 396;
			    else
				if (days_residue >= 143905)
				    probe = (days_residue >= 144270)? 395 : 394;
				else
				    probe = 393;
			else
			    if (days_residue >= 142444)
				if (days_residue >= 142809)
				    probe = (days_residue >= 143175)? 392 : 391;
				else
				    probe = 390;
			    else
				if (days_residue >= 141714)
				    probe = (days_residue >= 142079)? 389 : 388;
				else
				    probe = 387;
		    else
			if (days_residue >= 139157)
			    if (days_residue >= 140253)
				if (days_residue >= 140618)
				    probe = (days_residue >= 140983)? 386 : 385;
				else
				    probe = 384;
			    else
				if (days_residue >= 139522)
				    probe = (days_residue >= 139887)? 383 : 382;
				else
				    probe = 381;
			else
			    if (days_residue >= 138061)
				if (days_residue >= 138426)
				    probe = (days_residue >= 138792)? 380 : 379;
				else
				    probe = 378;
			    else
				if (days_residue >= 137331)
				    probe = (days_residue >= 137696)? 377 : 376;
				else
				    probe = 375;
		else
		    if (days_residue >= 132217)
			if (days_residue >= 134409)
			    if (days_residue >= 135504)
				if (days_residue >= 136235)
				    probe = (days_residue >= 136600)? 374 : 373;
				else
				    probe = (days_residue >= 135870)? 372 : 371;
			    else
				if (days_residue >= 134774)
				    probe = (days_residue >= 135139)? 370 : 369;
				else
				    probe = 368;
			else
			    if (days_residue >= 133313)
				if (days_residue >= 133678)
				    probe = (days_residue >= 134043)? 367 : 366;
				else
				    probe = 365;
			    else
				if (days_residue >= 132582)
				    probe = (days_residue >= 132948)? 364 : 363;
				else
				    probe = 362;
		    else
			if (days_residue >= 130026)
			    if (days_residue >= 131121)
				if (days_residue >= 131487)
				    probe = (days_residue >= 131852)? 361 : 360;
				else
				    probe = 359;
			    else
				if (days_residue >= 130391)
				    probe = (days_residue >= 130756)? 358 : 357;
				else
				    probe = 356;
			else
			    if (days_residue >= 128930)
				if (days_residue >= 129295)
				    probe = (days_residue >= 129660)? 355 : 354;
				else
				    probe = 353;
			    else
				if (days_residue >= 128199)
				    probe = (days_residue >= 128565)? 352 : 351;
				else
				    probe = 350;
	    else
		if (days_residue >= 118703)
		    if (days_residue >= 123086)
			if (days_residue >= 125277)
			    if (days_residue >= 126373)
				if (days_residue >= 127104)
				    probe = (days_residue >= 127469)? 349 : 348;
				else
				    probe = (days_residue >= 126738)? 347 : 346;
			    else
				if (days_residue >= 125643)
				    probe = (days_residue >= 126008)? 345 : 344;
				else
				    probe = 343;
			else
			    if (days_residue >= 124182)
				if (days_residue >= 124547)
				    probe = (days_residue >= 124912)? 342 : 341;
				else
				    probe = 340;
			    else
				if (days_residue >= 123451)
				    probe = (days_residue >= 123816)? 339 : 338;
				else
				    probe = 337;
		    else
			if (days_residue >= 120894)
			    if (days_residue >= 121990)
				if (days_residue >= 122355)
				    probe = (days_residue >= 122721)? 336 : 335;
				else
				    probe = 334;
			    else
				if (days_residue >= 121260)
				    probe = (days_residue >= 121625)? 333 : 332;
				else
				    probe = 331;
			else
			    if (days_residue >= 119799)
				if (days_residue >= 120164)
				    probe = (days_residue >= 120529)? 330 : 329;
				else
				    probe = 328;
			    else
				if (days_residue >= 119068)
				    probe = (days_residue >= 119433)? 327 : 326;
				else
				    probe = 325;
		else
		    if (days_residue >= 113955)
			if (days_residue >= 116146)
			    if (days_residue >= 117242)
				if (days_residue >= 117972)
				    probe = (days_residue >= 118338)? 324 : 323;
				else
				    probe = (days_residue >= 117607)? 322 : 321;
			    else
				if (days_residue >= 116511)
				    probe = (days_residue >= 116877)? 320 : 319;
				else
				    probe = 318;
			else
			    if (days_residue >= 115050)
				if (days_residue >= 115416)
				    probe = (days_residue >= 115781)? 317 : 316;
				else
				    probe = 315;
			    else
				if (days_residue >= 114320)
				    probe = (days_residue >= 114685)? 314 : 313;
				else
				    probe = 312;
		    else
			if (days_residue >= 111763)
			    if (days_residue >= 112859)
				if (days_residue >= 113224)
				    probe = (days_residue >= 113589)? 311 : 310;
				else
				    probe = 309;
			    else
				if (days_residue >= 112128)
				    probe = (days_residue >= 112494)? 308 : 307;
				else
				    probe = 306;
			else
			    if (days_residue >= 110667)
				if (days_residue >= 111033)
				    probe = (days_residue >= 111398)? 305 : 304;
				else
				    probe = 303;
			    else
				if (days_residue >= 109937)
				    probe = (days_residue >= 110302)? 302 : 301;
				else
				    probe = 300;
	else
	    if (days_residue >= 91310)
		if (days_residue >= 100441)
		    if (days_residue >= 104824)
			if (days_residue >= 107016)
			    if (days_residue >= 108112)
				if (days_residue >= 108842)
				    probe = (days_residue >= 109207)? 299 : 298;
				else
				    probe = (days_residue >= 108477)? 297 : 296;
			    else
				if (days_residue >= 107381)
				    probe = (days_residue >= 107746)? 295 : 294;
				else
				    probe = 293;
			else
			    if (days_residue >= 105920)
				if (days_residue >= 106285)
				    probe = (days_residue >= 106651)? 292 : 291;
				else
				    probe = 290;
			    else
				if (days_residue >= 105190)
				    probe = (days_residue >= 105555)? 289 : 288;
				else
				    probe = 287;
		    else
			if (days_residue >= 102633)
			    if (days_residue >= 103729)
				if (days_residue >= 104094)
				    probe = (days_residue >= 104459)? 286 : 285;
				else
				    probe = 284;
			    else
				if (days_residue >= 102998)
				    probe = (days_residue >= 103363)? 283 : 282;
				else
				    probe = 281;
			else
			    if (days_residue >= 101537)
				if (days_residue >= 101902)
				    probe = (days_residue >= 102268)? 280 : 279;
				else
				    probe = 278;
			    else
				if (days_residue >= 100807)
				    probe = (days_residue >= 101172)? 277 : 276;
				else
				    probe = 275;
		else
		    if (days_residue >= 95693)
			if (days_residue >= 97885)
			    if (days_residue >= 98980)
				if (days_residue >= 99711)
				    probe = (days_residue >= 100076)? 274 : 273;
				else
				    probe = (days_residue >= 99346)? 272 : 271;
			    else
				if (days_residue >= 98250)
				    probe = (days_residue >= 98615)? 270 : 269;
				else
				    probe = 268;
			else
			    if (days_residue >= 96789)
				if (days_residue >= 97154)
				    probe = (days_residue >= 97519)? 267 : 266;
				else
				    probe = 265;
			    else
				if (days_residue >= 96058)
				    probe = (days_residue >= 96424)? 264 : 263;
				else
				    probe = 262;
		    else
			if (days_residue >= 93502)
			    if (days_residue >= 94597)
				if (days_residue >= 94963)
				    probe = (days_residue >= 95328)? 261 : 260;
				else
				    probe = 259;
			    else
				if (days_residue >= 93867)
				    probe = (days_residue >= 94232)? 258 : 257;
				else
				    probe = 256;
			else
			    if (days_residue >= 92406)
				if (days_residue >= 92771)
				    probe = (days_residue >= 93136)? 255 : 254;
				else
				    probe = 253;
			    else
				if (days_residue >= 91675)
				    probe = (days_residue >= 92041)? 252 : 251;
				else
				    probe = 250;
	    else
		if (days_residue >= 82179)
		    if (days_residue >= 86562)
			if (days_residue >= 88753)
			    if (days_residue >= 89849)
				if (days_residue >= 90580)
				    probe = (days_residue >= 90945)? 249 : 248;
				else
				    probe = (days_residue >= 90214)? 247 : 246;
			    else
				if (days_residue >= 89119)
				    probe = (days_residue >= 89484)? 245 : 244;
				else
				    probe = 243;
			else
			    if (days_residue >= 87658)
				if (days_residue >= 88023)
				    probe = (days_residue >= 88388)? 242 : 241;
				else
				    probe = 240;
			    else
				if (days_residue >= 86927)
				    probe = (days_residue >= 87292)? 239 : 238;
				else
				    probe = 237;
		    else
			if (days_residue >= 84370)
			    if (days_residue >= 85466)
				if (days_residue >= 85831)
				    probe = (days_residue >= 86197)? 236 : 235;
				else
				    probe = 234;
			    else
				if (days_residue >= 84736)
				    probe = (days_residue >= 85101)? 233 : 232;
				else
				    probe = 231;
			else
			    if (days_residue >= 83275)
				if (days_residue >= 83640)
				    probe = (days_residue >= 84005)? 230 : 229;
				else
				    probe = 228;
			    else
				if (days_residue >= 82544)
				    probe = (days_residue >= 82909)? 227 : 226;
				else
				    probe = 225;
		else
		    if (days_residue >= 77431)
			if (days_residue >= 79622)
			    if (days_residue >= 80718)
				if (days_residue >= 81448)
				    probe = (days_residue >= 81814)? 224 : 223;
				else
				    probe = (days_residue >= 81083)? 222 : 221;
			    else
				if (days_residue >= 79987)
				    probe = (days_residue >= 80353)? 220 : 219;
				else
				    probe = 218;
			else
			    if (days_residue >= 78526)
				if (days_residue >= 78892)
				    probe = (days_residue >= 79257)? 217 : 216;
				else
				    probe = 215;
			    else
				if (days_residue >= 77796)
				    probe = (days_residue >= 78161)? 214 : 213;
				else
				    probe = 212;
		    else
			if (days_residue >= 75239)
			    if (days_residue >= 76335)
				if (days_residue >= 76700)
				    probe = (days_residue >= 77065)? 211 : 210;
				else
				    probe = 209;
			    else
				if (days_residue >= 75604)
				    probe = (days_residue >= 75970)? 208 : 207;
				else
				    probe = 206;
			else
			    if (days_residue >= 74143)
				if (days_residue >= 74509)
				    probe = (days_residue >= 74874)? 205 : 204;
				else
				    probe = 203;
			    else
				if (days_residue >= 73413)
				    probe = (days_residue >= 73778)? 202 : 201;
				else
				    probe = 200;
    else
	if (days_residue >= 36524)
	    if (days_residue >= 54786)
		if (days_residue >= 63917)
		    if (days_residue >= 68300)
			if (days_residue >= 70492)
			    if (days_residue >= 71588)
				if (days_residue >= 72318)
				    probe = (days_residue >= 72683)? 199 : 198;
				else
				    probe = (days_residue >= 71953)? 197 : 196;
			    else
				if (days_residue >= 70857)
				    probe = (days_residue >= 71222)? 195 : 194;
				else
				    probe = 193;
			else
			    if (days_residue >= 69396)
				if (days_residue >= 69761)
				    probe = (days_residue >= 70127)? 192 : 191;
				else
				    probe = 190;
			    else
				if (days_residue >= 68666)
				    probe = (days_residue >= 69031)? 189 : 188;
				else
				    probe = 187;
		    else
			if (days_residue >= 66109)
			    if (days_residue >= 67205)
				if (days_residue >= 67570)
				    probe = (days_residue >= 67935)? 186 : 185;
				else
				    probe = 184;
			    else
				if (days_residue >= 66474)
				    probe = (days_residue >= 66839)? 183 : 182;
				else
				    probe = 181;
			else
			    if (days_residue >= 65013)
				if (days_residue >= 65378)
				    probe = (days_residue >= 65744)? 180 : 179;
				else
				    probe = 178;
			    else
				if (days_residue >= 64283)
				    probe = (days_residue >= 64648)? 177 : 176;
				else
				    probe = 175;
		else
		    if (days_residue >= 59169)
			if (days_residue >= 61361)
			    if (days_residue >= 62456)
				if (days_residue >= 63187)
				    probe = (days_residue >= 63552)? 174 : 173;
				else
				    probe = (days_residue >= 62822)? 172 : 171;
			    else
				if (days_residue >= 61726)
				    probe = (days_residue >= 62091)? 170 : 169;
				else
				    probe = 168;
			else
			    if (days_residue >= 60265)
				if (days_residue >= 60630)
				    probe = (days_residue >= 60995)? 167 : 166;
				else
				    probe = 165;
			    else
				if (days_residue >= 59534)
				    probe = (days_residue >= 59900)? 164 : 163;
				else
				    probe = 162;
		    else
			if (days_residue >= 56978)
			    if (days_residue >= 58073)
				if (days_residue >= 58439)
				    probe = (days_residue >= 58804)? 161 : 160;
				else
				    probe = 159;
			    else
				if (days_residue >= 57343)
				    probe = (days_residue >= 57708)? 158 : 157;
				else
				    probe = 156;
			else
			    if (days_residue >= 55882)
				if (days_residue >= 56247)
				    probe = (days_residue >= 56612)? 155 : 154;
				else
				    probe = 153;
			    else
				if (days_residue >= 55151)
				    probe = (days_residue >= 55517)? 152 : 151;
				else
				    probe = 150;
	    else
		if (days_residue >= 45655)
		    if (days_residue >= 50038)
			if (days_residue >= 52229)
			    if (days_residue >= 53325)
				if (days_residue >= 54056)
				    probe = (days_residue >= 54421)? 149 : 148;
				else
				    probe = (days_residue >= 53690)? 147 : 146;
			    else
				if (days_residue >= 52595)
				    probe = (days_residue >= 52960)? 145 : 144;
				else
				    probe = 143;
			else
			    if (days_residue >= 51134)
				if (days_residue >= 51499)
				    probe = (days_residue >= 51864)? 142 : 141;
				else
				    probe = 140;
			    else
				if (days_residue >= 50403)
				    probe = (days_residue >= 50768)? 139 : 138;
				else
				    probe = 137;
		    else
			if (days_residue >= 47846)
			    if (days_residue >= 48942)
				if (days_residue >= 49307)
				    probe = (days_residue >= 49673)? 136 : 135;
				else
				    probe = 134;
			    else
				if (days_residue >= 48212)
				    probe = (days_residue >= 48577)? 133 : 132;
				else
				    probe = 131;
			else
			    if (days_residue >= 46751)
				if (days_residue >= 47116)
				    probe = (days_residue >= 47481)? 130 : 129;
				else
				    probe = 128;
			    else
				if (days_residue >= 46020)
				    probe = (days_residue >= 46385)? 127 : 126;
				else
				    probe = 125;
		else
		    if (days_residue >= 40907)
			if (days_residue >= 43098)
			    if (days_residue >= 44194)
				if (days_residue >= 44924)
				    probe = (days_residue >= 45290)? 124 : 123;
				else
				    probe = (days_residue >= 44559)? 122 : 121;
			    else
				if (days_residue >= 43463)
				    probe = (days_residue >= 43829)? 120 : 119;
				else
				    probe = 118;
			else
			    if (days_residue >= 42002)
				if (days_residue >= 42368)
				    probe = (days_residue >= 42733)? 117 : 116;
				else
				    probe = 115;
			    else
				if (days_residue >= 41272)
				    probe = (days_residue >= 41637)? 114 : 113;
				else
				    probe = 112;
		    else
			if (days_residue >= 38715)
			    if (days_residue >= 39811)
				if (days_residue >= 40176)
				    probe = (days_residue >= 40541)? 111 : 110;
				else
				    probe = 109;
			    else
				if (days_residue >= 39080)
				    probe = (days_residue >= 39446)? 108 : 107;
				else
				    probe = 106;
			else
			    if (days_residue >= 37619)
				if (days_residue >= 37985)
				    probe = (days_residue >= 38350)? 105 : 104;
				else
				    probe = 103;
			    else
				if (days_residue >= 36889)
				    probe = (days_residue >= 37254)? 102 : 101;
				else
				    probe = 100;
	else
	    if (days_residue >= 18262)
		if (days_residue >= 27393)
		    if (days_residue >= 31776)
			if (days_residue >= 33968)
			    if (days_residue >= 35064)
				if (days_residue >= 35794)
				    probe = (days_residue >= 36159)? 99 : 98;
				else
				    probe = (days_residue >= 35429)? 97 : 96;
			    else
				if (days_residue >= 34333)
				    probe = (days_residue >= 34698)? 95 : 94;
				else
				    probe = 93;
			else
			    if (days_residue >= 32872)
				if (days_residue >= 33237)
				    probe = (days_residue >= 33603)? 92 : 91;
				else
				    probe = 90;
			    else
				if (days_residue >= 32142)
				    probe = (days_residue >= 32507)? 89 : 88;
				else
				    probe = 87;
		    else
			if (days_residue >= 29585)
			    if (days_residue >= 30681)
				if (days_residue >= 31046)
				    probe = (days_residue >= 31411)? 86 : 85;
				else
				    probe = 84;
			    else
				if (days_residue >= 29950)
				    probe = (days_residue >= 30315)? 83 : 82;
				else
				    probe = 81;
			else
			    if (days_residue >= 28489)
				if (days_residue >= 28854)
				    probe = (days_residue >= 29220)? 80 : 79;
				else
				    probe = 78;
			    else
				if (days_residue >= 27759)
				    probe = (days_residue >= 28124)? 77 : 76;
				else
				    probe = 75;
		else
		    if (days_residue >= 22645)
			if (days_residue >= 24837)
			    if (days_residue >= 25932)
				if (days_residue >= 26663)
				    probe = (days_residue >= 27028)? 74 : 73;
				else
				    probe = (days_residue >= 26298)? 72 : 71;
			    else
				if (days_residue >= 25202)
				    probe = (days_residue >= 25567)? 70 : 69;
				else
				    probe = 68;
			else
			    if (days_residue >= 23741)
				if (days_residue >= 24106)
				    probe = (days_residue >= 24471)? 67 : 66;
				else
				    probe = 65;
			    else
				if (days_residue >= 23010)
				    probe = (days_residue >= 23376)? 64 : 63;
				else
				    probe = 62;
		    else
			if (days_residue >= 20454)
			    if (days_residue >= 21549)
				if (days_residue >= 21915)
				    probe = (days_residue >= 22280)? 61 : 60;
				else
				    probe = 59;
			    else
				if (days_residue >= 20819)
				    probe = (days_residue >= 21184)? 58 : 57;
				else
				    probe = 56;
			else
			    if (days_residue >= 19358)
				if (days_residue >= 19723)
				    probe = (days_residue >= 20088)? 55 : 54;
				else
				    probe = 53;
			    else
				if (days_residue >= 18627)
				    probe = (days_residue >= 18993)? 52 : 51;
				else
				    probe = 50;
	    else
		if (days_residue >= 9131)
		    if (days_residue >= 13514)
			if (days_residue >= 15705)
			    if (days_residue >= 16801)
				if (days_residue >= 17532)
				    probe = (days_residue >= 17897)? 49 : 48;
				else
				    probe = (days_residue >= 17166)? 47 : 46;
			    else
				if (days_residue >= 16071)
				    probe = (days_residue >= 16436)? 45 : 44;
				else
				    probe = 43;
			else
			    if (days_residue >= 14610)
				if (days_residue >= 14975)
				    probe = (days_residue >= 15340)? 42 : 41;
				else
				    probe = 40;
			    else
				if (days_residue >= 13879)
				    probe = (days_residue >= 14244)? 39 : 38;
				else
				    probe = 37;
		    else
			if (days_residue >= 11322)
			    if (days_residue >= 12418)
				if (days_residue >= 12783)
				    probe = (days_residue >= 13149)? 36 : 35;
				else
				    probe = 34;
			    else
				if (days_residue >= 11688)
				    probe = (days_residue >= 12053)? 33 : 32;
				else
				    probe = 31;
			else
			    if (days_residue >= 10227)
				if (days_residue >= 10592)
				    probe = (days_residue >= 10957)? 30 : 29;
				else
				    probe = 28;
			    else
				if (days_residue >= 9496)
				    probe = (days_residue >= 9861)? 27 : 26;
				else
				    probe = 25;
		else
		    if (days_residue >= 4383)
			if (days_residue >= 6574)
			    if (days_residue >= 7670)
				if (days_residue >= 8400)
				    probe = (days_residue >= 8766)? 24 : 23;
				else
				    probe = (days_residue >= 8035)? 22 : 21;
			    else
				if (days_residue >= 6939)
				    probe = (days_residue >= 7305)? 20 : 19;
				else
				    probe = 18;
			else
			    if (days_residue >= 5478)
				if (days_residue >= 5844)
				    probe = (days_residue >= 6209)? 17 : 16;
				else
				    probe = 15;
			    else
				if (days_residue >= 4748)
				    probe = (days_residue >= 5113)? 14 : 13;
				else
				    probe = 12;
		    else
			if (days_residue >= 2191)
			    if (days_residue >= 3287)
				if (days_residue >= 3652)
				    probe = (days_residue >= 4017)? 11 : 10;
				else
				    probe = 9;
			    else
				if (days_residue >= 2556)
				    probe = (days_residue >= 2922)? 8 : 7;
				else
				    probe = 6;
			else
			    if (days_residue >= 1095)
				if (days_residue >= 1461)
				    probe = (days_residue >= 1826)? 5 : 4;
				else
				    probe = 3;
			    else
				if (days_residue >= 365)
				    probe = (days_residue >= 730)? 2 : 1;
				else
				    probe = 0;

    years = probe;  /* needed to test for leap year below */
    greg->year = probe;
    days_residue -= dy[probe]; 

    // Figure out exact year by adding years corresponing to # of gregs
    // greg->year += gregs*400;
    // hard-wired multiply by 400 */
    templl = gregs;
    greg->year += templl << 8;
    greg->year += templl << 7;
    greg->year += templl << 4;
    greg->year++;  // Correct for year starting at 1AD 

    // Table lookup to calculate year and month
    i = leap_year[years]; // cache result of leap year test
    greg->month = (Months)( ((i))? day_to_leap_month[days_residue] : day_to_month[days_residue]);
    greg->day = ((i))? day_to_leap_day[days_residue] : day_to_day[days_residue];

    // reapportion  residue seconds to hours, minutes and seconds
    greg->hour = seconds_residue / 3600;
    seconds_residue = seconds_residue % 3600;
    greg->minute = seconds_residue / 60;
    greg->second = seconds_residue % 60;
    
    return GOk;
}


/*************************************************************************/
/******************************* THE  END ********************************/
/*************************************************************************/


