/****************************************************************************
 *   ggraph.cpp : 
 *   		impl file for GFC Graph.
 *
 *   Change Log :
 *		 Oct 27, 1998 Initial version,  Liujian Qian
 *
 *   $Id: ggraph.cpp,v 1.12 1999/02/15 09:13:43 qian Exp $
 ***************************************************************************/
#include <iostream.h>
#include "ggraph.h"	

GNode::GNode()
{
    u.i = v.i = w.i = x.i = y.i = z.i = 0;
    next = 0;
    wire_list = 0;
}

void
GNode::removeWire(GWire* wire)
{
    if(wire==wire_list) wire_list = wire_list->next;
    else {
	GWire *	wr, *prev=0;
	wr = prev = wire_list;
	while(wr) {
	    prev = wr;
	    wr = wr->next;
	    if(wr==wire) { prev->next = wire->next; return; }
	}
    }
}

GWire::GWire()
{
    u.i = v.i = w.i = 0;
    orig = dest = 0;
    tip  = 0;
    next = 0;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
/// GGraph

GGraph::GGraph		(bool _directed, 
			 int _next_node_id)
{
    flags.directed = _directed;
    next_node_id = _next_node_id;
    node_list = 0;
}

GGraph::~GGraph		()
{
    freeWires();
    freeNodes();
};


void 
GGraph::addNode		(GNode* nd)
{
    if(!nd) return;
    nd->id = next_node_id++;
    nd->next = node_list;  
    node_list = nd;
    n_nodes ++;
}

void
GGraph::addNode		(GNode* nd, int id)
{
    if(!nd) return;    		if(id<0); //cry
    nd->id = id;        	
    nd->next = node_list;
    node_list = nd;    		
    n_nodes ++;
}

void
GGraph::delNode		(GNode* nd)
{
    if(!nd || !node_list) return;
    GNode* tip;
    
    //remove wires connected to it
    for(GWire* w=nd->wire_list; w; ) {
	GWire* next = w->next;
	tip = w->tip;
	disconnect(nd, tip);
	w = next;
    }
    //pick it off the node list
    if(node_list == nd) node_list = nd->next;
    else {
	GNode *n, *prev=0;
	n = prev = node_list;
	while(n) {
	    prev = n;
	    n = n->next;
	    if(n==nd) { prev->next = nd->next; break; }
	}
    }
    delete nd; n_nodes --;
}

void
GGraph::delNode		(int id)
{
    GNode *nd = lookup(id);
    if(!nd) return;
    delNode(nd);
}


GWire*
GGraph::_connect1	(GNode* orig, GNode* dest, float weight)
{
    if(!orig || !dest) 		return 0;
    if(isNeighbor(orig, dest)) return 0;
    
    //generate a new wire
    GWire *w = new GWire;
    w->tip = dest; w->w.f = weight;
    w->orig = orig->id, w->dest = dest->id;
    //add to the wire list of orig
    w->next = orig->wire_list;
    orig->wire_list = w; n_wires ++;

    return w;
}

GWire*
GGraph::connect		(GNode* orig, GNode* dest, float weight)
{
    return _connect1(orig, dest, weight);
    
    /* only one wire is physically created even for undirected graphs
    if(! flags.directed )  //add another wire
	return _connect1(dest, orig, weight);
    */
}

GWire*
GGraph::connect		(int orig_id, int dest_id, float weight)
{
    GNode *o = lookup(orig_id), *d = lookup(dest_id);
    if(o && d) return connect(o,d, weight);
    else return 0;
}

//really removes only ONE wire from orig->dest.
void
GGraph::_disconnect1	(GNode* orig, GNode* dest)
{
    if(!orig || !dest) return;
    
    int oid = orig->id, did = dest->id;
    
    for(GWire* w = orig->wire_list; w; w=w->next) 
	if(w->orig==oid && w->dest==did) { //gotcha
	    orig->removeWire(w);
	    n_wires--;	return;
	}
}

void
GGraph::disconnect	(GNode* orig, GNode* dest)
{
    _disconnect1(orig, dest);
    if(!flags.directed)
	_disconnect1(dest, orig);
}

void
GGraph::disconnect	(int orig_id, int dest_id)
{
    GNode *o = lookup(orig_id), *d = lookup(dest_id);
    if(o && d) disconnect(o,d);    
}

void	
GGraph::freeNodes()
{}

void	
GGraph::freeWires()
{}

GNode*	
GGraph::lookup		(int id) const
{
    for(GNode* n = node_list; n; n=n->next) 
	if(id == n->id) return n;
    return (GNode*) 0;
}

bool
GGraph::isRoot		(GNode* nd) const
{
    if(!nd) return false;
     
    //can this be improved???
   
    for(GNode* n=node_list; n; n=n->next) {
	for(GWire* w=n->wire_list; w; w=w->next) {
	    if(w->tip == nd) return false;
	}
    }

    return true;
}

bool
GGraph::isRoot		(int id) const
{
    return isRoot( lookup(id) );
}

bool
GGraph::isLeaf		(GNode* nd) const
{
    if(!nd) return false;
    //as long as there is no outgoing wire this is leaf.
    if(nd->wire_list) return false;
    GNode* n = node_list;
    GWire *w;
    while(n) {
	w = n->wire_list;
	while(w) {
	    if(w->tip == nd) return true;
	    w = w->next;
	}
	n = n->next;
    }

    return false;  //this must be an isolated node
}

bool	
GGraph::isLeaf		(int id) const
{
    return isLeaf( lookup(id) );
}

bool
GGraph::isIsolated	(int node_id)
{
    GNode *nd = lookup(node_id);
    return isIsolated(nd);
}

bool
GGraph::isIsolated	(GNode* nd)
{
    if(!nd)
	return true;

    return (nd->wire_list ==0 && !isLeaf(nd));
}
    
bool
GGraph::isNeighbor	(GNode* nd1, GNode* nd2)
{
    //!!! potential glitch: shall we test both
    //!!! connections for undirected graphs? 

    for(GWire* w=nd1->wire_list; w; w=w->next) {
	if(w->tip == nd2) return true;
    }
    return false;
}

bool
GGraph::isNeighbor	(int id1, int id2)
{
    GNode *orig=lookup(id1), *dest=lookup(id2);
    if(!orig || !dest) 		return false;
    return isNeighbor(orig, dest);    
}

bool
GGraph::isConnected	(GNode* nd1, GNode* nd2)
{
    if(isNeighbor(nd1, nd2)) return true;
    //else see if we have a path between them.
    return false;
}

bool
GGraph::isConnected	(int id1, int id2)
{
    GNode *orig=lookup(id1), *dest=lookup(id2);
    if(!orig || !dest) 		return false;
    return isConnected(orig, dest);    
}

ostream&    operator<<(ostream& os, const GGraph& g)
{
    os<<"Nodes:\n";
    GNode* nd;
    for(nd=g.node_list; nd; nd=nd->next) {
	os<<"\t"<<nd->id<<", "<<nd->name<<"\n";
    }
    os<<"Wires:\n";
    //this is how to loop over all the wires
    for(nd=g.node_list; nd; nd=nd->next) {
	for(GWire* w=nd->wire_list; w; w=w->next) 
	    os<<"\t("<<w->orig<<", "<<w->dest<<")\n";
    }
    return os;
}
