/****************************************************************************
 *   gops.cpp : 
 *   		main file for GFC Ordered Point Sets class.
 *
 *   Change Log :
 *		 July 1, 1998 Initial version,  Liujian Qian
 *
 *   $Id: gops.cpp,v 1.13 1999/02/08 05:22:39 qian Exp $
 ***************************************************************************/
#include <string.h>
#include <strstream.h>
#include "gops.h"
#include "gadt_basic.h"
#include "gadt_segment.h"
#include "gadt_point.h"
#include "gadt_circle.h"

GOps::GOps()
{
    parts = 0, coord = 0;
    init();
}

GOps::GOps(const GOps& o)
{
    parts=0, coord=0;
    this->operator=(o);
}

GOps&
GOps::operator= (const GOps& a)
{
    n_points = a.n_points;
    n_parts = a.n_parts;
    extendPartsTo(n_parts, 0 /*keep not old stuff*/);  
    extendPointTo(n_points, 0);
    
    memcpy((char*)parts, (char*)a.parts, sizeof(int)*n_parts);
    memcpy((char*)coord, (char*)a.coord, COORD_SZ(n_points));

    xl=a.xl, yl=a.yl, xh=a.xh, yh=a.yh;
    return *this;
}

GOps::~GOps()
{
    if(parts) delete[] parts;
    if(coord) delete[] coord;
}


Result
GOps::init (void)
{
    if(parts) delete[] parts;
    if(coord) delete[] coord;
    parts = 0, coord = 0;
    n_parts = n_points = 0;
    xl=yl=xh=yh=InvalidCoord;
    parts_cap=point_cap=0;
    return GOk;
}
    
uint32
GOps::getPSize () const 
{
    return  (sizeof(xl)*4 +   //mbr
	     sizeof(n_parts) + sizeof(n_points) + 
	     sizeof(int)*n_parts + //parts array
	     sizeof(Coord)*PolyDim*n_points); //coord array
}

int
GOps::PointCapacity(int n_pnts)
{
    return ( (n_pnts>>point_inc) + 1 ) << point_inc;
}

int 
GOps::PartsCapacity(int n_p)
{
    return ( (n_p>>parts_inc) + 1 ) << parts_inc;
}

Result
GOps::input (const char* in) 
{
    GAssert(in);
    init();
    
    int  i,cur =0;
    char word[68];
    
    getNextWord(in, word, cur);
    n_parts = atol(word);

    if(n_parts<0 || n_parts>=max_parts)
	msgFail("invalid number of parts in ASCII OPS string");
    parts_cap = PartsCapacity(n_parts);
    GAssert(parts = new int[parts_cap]);
    
    for(i=0; i<n_parts; i++) {
	memset(word, 0, 68);
	getNextWord(in, word, cur);
	parts[i] = atol(word);
	n_points += parts[i];
    }

    point_cap = PointCapacity(n_points);
    GAssert(coord = new Coord[point_cap*PolyDim]);

    for(i=0; i<n_points; i++) {
	memset(word, 0, 68); 
	if (!getNextWord(in, word, cur)) 
	    msgFail("invalid ASCII OPS string") ;
	coord[i*2+0] = strtod(word, NULL);
	memset(word, 0, 68); 
	if (!getNextWord(in, word, cur)) 
	    msgFail("invalid ASCII OPS string") ;
	coord[i*2+1] = strtod(word, NULL);
    }
    return GOk;
}


Result
GOps::output (char*& out) 
{
  int exp_len=0, i;
  
  // assume 8 bytes for ascii coordinate
  exp_len = 12 + n_parts*6 + n_points*PolyDim*(8+2);
  bp.extendTo(exp_len);
  bp.recycle();
  char *p = (char*) bp.ptr();
  ostrstream os(p, exp_len);
  
  os<<n_parts<<" ";
  for(i=0; i<n_parts; i++) os<<parts[i]<<" ";

  //output actual array of vetexs
  for(i=0; i<n_points; i++) os<<coord[i*PolyDim+0]<<" "<<coord[i*PolyDim+1]<<" ";

  out = p;
  return GOk;
}


Result
GOps::pack (DataPipe& p ) 
{
    int   t;
    double d;

    //pack the mbr
    if(GEndian::big) 
	{
	    d= xl;	flipDouble(&d);
	    p.append(CONPTR(&d), sizeof(xl));
	    d = yl;	flipDouble(&d);
	    p.append(CONPTR(&d), sizeof(yl));
	    d = xh;	flipDouble(&d);
	    p.append(CONPTR(&d), sizeof(xh));
	    d = yh; 	flipDouble(&d);
	    p.append(CONPTR(&d), sizeof(yh));
	    //pack n_parts/n_points
	    t = n_parts; flipInt(&t);
	    p.append(CONPTR(&t), sizeof(t));
	    t = n_points; flipInt(&t);
	    p.append(CONPTR(&t), sizeof(t));
	} 
    else  //we are on a little-endian system
	{
	    p.append(CONPTR(&xl), sizeof(xl));
	    p.append(CONPTR(&yl), sizeof(yl));
	    p.append(CONPTR(&xh), sizeof(xh));
	    p.append(CONPTR(&yh), sizeof(yh));
	    //pack n_parts/n_points
	    p.append(CONPTR(&n_parts), sizeof(uint32));
	    p.append(CONPTR(&n_points), sizeof(uint32));
	}
    
    //pack part array
    for(int i=0; i<n_parts; i++) {
	if(GEndian::big) {
	    t = parts[i];
	    flipInt(&t);
	}
	else t=parts[i];
	p.append(CONPTR(&t), sizeof(t));
    }
    //pack point coordinates array
    if(GEndian::big) {
	double ux, uy;
	for(int i=0; i<n_points; i++) {
	    ux = coord[i*PolyDim+0]; 
	    uy = coord[i*PolyDim+1]; 
	    flipDouble(&ux);
	    flipDouble(&uy);
	    p.append(CONPTR(&ux), sizeof(Coord));
	    p.append(CONPTR(&uy), sizeof(Coord));
	}
    } else {
	for(int i=0; i<n_points; i++) {
	    p.append(CONPTR(&coord[i*PolyDim]), sizeof(Coord));
	    p.append(CONPTR(&coord[i*PolyDim+1]), sizeof(Coord));
	}
    }

    return GOk;
}

Result
GOps::unpack (DataPipe& p) 
{
    GAssert(!p.eop());
    //unpack mbr
    p.fetch((char*)&xl, sizeof(Coord));
    p.fetch((char*)&yl, sizeof(Coord));
    p.fetch((char*)&xh,  sizeof(Coord));
    p.fetch((char*)&yh,  sizeof(Coord));

    if(GEndian::big) {
	flipDouble(&xl);
	flipDouble(&yl);
	flipDouble(&xh);
	flipDouble(&yh);
    }

    //unpack n_parts/n_points
    p.fetch((char*)&n_parts,  sizeof(n_parts));        
    p.fetch((char*)&n_points, sizeof(n_points));
    if(GEndian::big) {
	flipInt(&n_parts);
	flipInt(&n_points);
    }

    if(n_parts==0)  msgFail("no parts in the OPS");
    if(n_points==0) msgFail("no point in the OPS");
    
    GAssert( extendPartsTo(n_parts, 0) == GOk);
    GAssert( extendPointTo(n_points,0) == GOk);

    //unpack parts array
    p.fetch((char*)parts, sizeof(int)*n_parts);
    if(GEndian::big) 
	for(int i=0; i<n_parts; i++) flipInt(&parts[i]);
    //unpack coord array
    p.fetch((char*)coord, COORD_SZ(n_points));
    if(GEndian::big) 
	for(int i=0; i<n_points*PolyDim; i++) {
	    flipDouble(&coord[i]);
	}
    
    return GOk;
}

Result
GOps::mbr(GRect& r) const
{
    r.set(xl,yl,xh,yh);
    return GOk;
}

Result
GOps::calcMbr(GRect& r) 
{
    xl = xh = coord[0], yl=yh=coord[1];
    for(int i=1, x=0, y=0; i<n_points; i++) {
	x=i*PolyDim; 
	y=x+1;
	if(xl>coord[x]) xl = coord[x];
	if(xh<coord[x]) xh = coord[x];
	if(yl>coord[y]) yl = coord[y];
	if(yh<coord[y]) yh = coord[y];
    }
    r.set(xl,yl,xh,yh);
    return GOk;
}

void
GOps::setMbr(const GRect& r)
{
    xl = r.getXL(), yl = r.getYL();
    xh = r.getXH(), yh = r.getYH();
}

    
Result
GOps::scale      (double xf, double yf, double )
{
    for(int i=0; i<n_points; i++)
	coord[i*PolyDim] *= xf, coord[i*PolyDim+1] *= yf;
    return GOk;
}

Result
GOps::translate  (double xf, double yf, double )
{
    for(int i=0; i<n_points; i++)
	coord[i*PolyDim] += xf, coord[i*PolyDim+1] += yf;
    return GOk;
}    

Result
GOps::rotate	   (double, bool)
{
    return GOk;
}

Result
GOps::transform  (const GMatrix&)
{
    return GOk;
}

void
GOps::set(int n_pts, int n_pnts, const int* parts_a, const Coord* coo)
{
    extendPartsTo(n_pts, false /* does not keep old ones*/);
    extendPointTo(n_pnts, false);
    n_parts = n_pts;
    n_points = n_pnts;
    memcpy((char*)parts, (char*)parts_a, sizeof(int)*n_pts);
    memcpy((char*)coord, (char*)coo, COORD_SZ(n_pnts));
}

    

Result	
GOps::addPart (int n_pnts, Coord* c)
{
    extendPartsTo(n_parts+1, true);
    parts[n_parts] = n_pnts;
    n_parts ++;

    extendPointTo(n_points+n_pnts, true);
    memcpy((char*)coord+COORD_SZ(n_points), c, COORD_SZ(n_pnts));
    n_points += n_pnts;

    return GOk;    
}

int
GOps::getPart (int nth, Coord*& cp)
{
    if(nth<0 || nth>=n_parts) {
	cp = 0;
	return 0;
    }
    int count=0;
    for(int i=0; i<nth; i++) count+= parts[i];
    
    cp = (Coord*) &coord[count*PolyDim];
    return parts[nth];
}

int GOps::getPartSize(int nth) 
{
    if(nth<0 || nth>=n_parts) {
	msgWarn("out of range in parts");
	return 0;
    } else return parts[nth];
}

double
GOps::totalLength() const
{
    register double len=0, *coo, x0,y0,x1,y1;
    int count=0;

    for(int j=0; j<n_parts; j++) {
	coo = &coord[count*PolyDim];
	x0=coo[0], y0=coo[1];
	for(int i=1; i<parts[j]; i++) {
	    x1 = coo[i*PolyDim], y1=coo[i*PolyDim+1];
	    len += hypot(x1-x0, y1-y0);
	    x0 = x1, y0=y1;
	}
	count += parts[j];
    }

    return len;
}

double
GOps::partLength(int nth) const
{
    register double* coo, len=0, x0,y0,x1,y1;
    int count =0, i;
    for(i=0; i<nth; i++) count+= parts[i];
    coo = &coord[count*PolyDim];
    x0=coo[0], y0=coo[1];
    for(i=1; i<parts[nth]; i++) {
	x1 = coo[i*PolyDim], y1 = coo[i*PolyDim+1];
	len += hypot(x1-x0, y1-y0);
    }
    return len;
}


int
GOps::intersects(int n_pnts, Coord* coo) const
{
    GSegment seg_o, seg_i;

    double x0,y0,x1,y1;
    int count=0, i, j, k;
    for(i=0; i<n_parts; i++) {
	x0 = coord[count*2], y0=coord[count*2+1];
	for(j=1; j<parts[i]; j++) {
	    x1=coord[(count+j)*2], y1=coord[(count+j)*2+1];
	    seg_o.set(x0,y0,x1,y1);
	    x0 = coo[0], y0=coo[1];
	    for(k=1; k<n_pnts; k++) {
		x1=coo[k*2], y1=coo[k*2+1];
		seg_i.set(x0,y0,x1,y1);
		if(seg_o.intersects(seg_i)) return true;
		x0 = x1, y0=y1;
	    }
	    x0=seg_o.getX2(), y0=seg_o.getY2();
	}
	count += parts[i];
    }

    return false;
}	
    
int		
GOps::intersects(const GSegment& seg_in) const
{
    GPoint pnt0, pnt1;
    GSegment seg;
    double* coo=0;
    int j=0, count=0;
    for(j=0; j<n_parts; j++) {
	coo = &coord[count*PolyDim];
	pnt0.set(coo[0], coo[1]);
	int i=0;
	while(++i<parts[j]){ 
	    pnt1.set(coo[i*PolyDim], coo[i*PolyDim+1]);
	    if(pnt1.x == pnt0.x && pnt1.y == pnt0.y) continue;

	    //form a segment
	    seg.set(pnt0, pnt1);
	
	    //check if it intersects the given segment.
	    if(seg.intersects(seg_in)) return 1;
	    
	    //check next segment.
	    pnt0 = pnt1;
	}
    }
    
    return 0;
}

int
GOps::intersects(const GRect& r) const
{
    GRect _r;
    mbr(_r);
    if(! r.intersects(_r)) return 0;

    GSegment seg;
    int count=0;
    double x0,y0,x1,y1;
    //loop through all segments of all parts
    for(int i=0; i<n_parts; i++) {
	x0=coord[count*PolyDim], y0=coord[count*PolyDim+1];
	for(int j=1; j<parts[i]; j++) {
	    x1=coord[(count+j)*PolyDim], y1=coord[(count+j)*PolyDim+1];
	    seg.set(x0,y0,x1,y1);
	    if(seg.intersects(r)) return 1;
	    x0=x1,y0=y1;
	}
	count += parts[i];
    }
    return 0;
}

int
GOps::intersects(const GCircle& c) const
{
    if(isInside(c)) return 0;
    int count=0;
    double x0,y0,x1,y1;

    GSegment seg;
    //loop through all segments of all parts
    for(int i=0; i<n_parts; i++) {
	x0=coord[count*PolyDim], y0=coord[count*PolyDim+1];
	for(int j=1; j<parts[i]; j++) {
	    x1=coord[(count+j)*PolyDim], y1=coord[(count+j)*PolyDim+1];
	    seg.set(x0,y0,x1,y1);
	    if(seg.intersects(c)) return 1;
	    x0=x1,y0=y1;
	}
	count += parts[i];
    }
    return 0;
}


int
GOps::intersects(const GOps& other) const
{
    int count=0;
    for(int i=0; i<n_parts; i++) {
	if(other.intersects(parts[i], &coord[count*PolyDim])) 	
	    return 1;
	count+=parts[i];
    }
    return 0;
}


int
GOps::isInside(const GRect& r) const
{
    GPoint pnt;
    GRect  mbr(xl,yl,xh,yh);
    if(r.covers(mbr)) return 1;
    int count=0;

    for(int j=0; j<n_parts; j++) {
	for(int i=0; i<parts[j]; i++) {
	    pnt.set(coord[(count+i)*PolyDim], coord[(count+i)*PolyDim+1]);
	    if(!pnt.isInside(r)) return 0;
	}
	count += parts[j];
    }
    return 1;
}


int	
GOps::isInside(const GCircle& c) const
{
    GPoint pnt;
    int count=0;

    for(int j=0; j<n_parts; j++) {
	for(int i=0; i<parts[j]; i++) {
	    pnt.set(coord[(count+i)*PolyDim], coord[(count+i)*PolyDim+1]);
	    if(!pnt.isInside(c)) return 1;
	}
	count += parts[j];
    }
    return 1;
}


Result
GOps::extendPartsTo(int n, bool keep)
{
    if(parts_cap >= (unsigned)n) return GOk;  //still have enough space

    int *tmp, new_cap;
    new_cap = PartsCapacity(n);
    tmp = new int[new_cap];
    if(!tmp) msgFail("can't allocate parts buffer");
    
    memset((char*)tmp, 0, sizeof(int)*new_cap);
    if(keep && parts) memcpy((char*)tmp, (char*)parts, sizeof(int)*n_parts);
    if(parts) delete[] parts;
    parts = tmp;
    parts_cap = new_cap;
    return GOk;
}


Result
GOps::extendPointTo(int n, bool keep)
{
    if(point_cap >=(unsigned)n) return GOk;  //still have enough space

    int new_cap;
    double *tmp=0;

    new_cap = PointCapacity(n);

    tmp = new Coord[new_cap*PolyDim];
    if(!tmp) msgFail("can't allocate coordinates buffer");
    
    memset((char*)tmp, 0, sizeof(Coord)*new_cap);
    if(keep && coord) memcpy((char*)tmp, (char*)coord, COORD_SZ(n_points));
    if(coord) delete[] coord;
    coord = tmp;
    point_cap = new_cap;
    return GOk;
}

ostream&  operator<<(ostream& co, const GOps& o)
{
    int i;
    co<<"(OPS: n_points="<<o.n_points<<", n_parts="<<o.n_parts<<"<";
    for(i=0; i<o.n_parts; i++) co<<o.parts[i]<<",";
    co<<">\n\t<";
    for(i=0; i<o.n_points; i++){
	co<<o.coord[i*PolyDim]<<","<<o.coord[i*PolyDim+1]<<", ";
	if(i%7==0) co<<"\n\t";
    }
    return co<<">\n";
}

    
