/****************************************************************************
 *   voronoi.cpp : 
 *   		  A wrap-up of Fortune's sweep algorithm.
 *
 *   Change Log :
 *		  Feb 13, 1998 Initial version,  Liujian Qian
 *
 *   $Id: gvoronoi.cpp,v 1.7 1998/12/02 04:03:43 qian Exp $
 ***************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <iostream.h>
#include <malloc.h>
#include "gvoronoi.h"
#include "gcxx.h"

GVoronoi::GVoronoi()
{
}

GVoronoi::~GVoronoi()
{
    if (sites) delete[] sites;

    char* p=0; mem_keeper.seek(0);
    for(unsigned int i=0; i<mem_keeper.size()/sizeof(char*); i++) {
	mem_keeper.fetch((char*)&p, sizeof(char*));
	free(p);
    }

}

void GVoronoi::plotInit()
{
    float dx,dy,d;

    dy = ymax - ymin;
    dx = xmax - xmin;
    d = ( dx > dy ? dx : dy) * 1.1;
#if 0
    pxmin = xmin - (d-dx)/2.0;
    pxmax = xmax + (d-dx)/2.0;
    pymin = ymin - (d-dy)/2.0;
    pymax = ymax + (d-dy)/2.0;
#else
    pxmin = xmin-5;
    pxmax = xmax+5;
    pymin = ymin-5;
    pymax = ymax+5;
#endif

    cradius = (pxmax - pxmin)/350.0;
    n_real_edges = 0;
}

//this is where the edge is cliped for display 
int GVoronoi::outputEdge(VEdge*e, 
			 double& x1, double& y1, double& x2, double& y2)
{
    int res = clipLine(e, x1, y1, x2, y2);
    if(res) { 
	edges<<x1<<y1<<x2<<y2;
	n_real_edges ++;
#if 0
	printf("e %d", e->edgenbr);
	printf(" %d ", e->ep[0] != (struct Site *)NULL ? e->ep[0]->sitenbr : -1);
	printf("%d\n", e->ep[1] != (struct Site *)NULL ? e->ep[1]->sitenbr : -1);
#endif
	if(e->ep[0]) {
	    end_points<<e->ep[0]->coord.x<<e->ep[0]->coord.y;
	}
	if(e->ep[1]) {
	    end_points<<e->ep[1]->coord.x<<e->ep[1]->coord.y;
	}
	return 1;
    }
    else return 0;
}

void 	GVoronoi::addToDiagram(VEdge* e)
{
    if(e) e->next = diagram_edges;
    diagram_edges = e;
}

int GVoronoi::clipLine(VEdge* e, double& x1, double& y1, double& x2, double& y2)
{
    Site *s1, *s2;

    if(e -> a == 1.0 && e ->b >= 0.0)    {	
	s1 = e -> ep[1];
	s2 = e -> ep[0];
    }
    else 	{	
	s1 = e -> ep[0];
	s2 = e -> ep[1];
    };

    if(e -> a == 1.0)	{
	y1 = pymin;
	if (s1!=(struct Site *)NULL && s1->coord.y > pymin)
	    y1 = s1->coord.y;
	if(y1>pymax) return 0;
	x1 = e -> c - e -> b * y1;
	y2 = pymax;
	if (s2!=(struct Site *)NULL && s2->coord.y < pymax) 
	    y2 = s2->coord.y;
	if(y2<pymin) return(0);
	x2 = e -> c - e -> b * y2;
	if ((x1> pxmax & x2>pxmax) | (x1<pxmin&x2<pxmin)) return 0;
	if(x1> pxmax)
	    {	x1 = pxmax; y1 = (e -> c - x1)/e -> b;};
	if(x1<pxmin)
	    {	x1 = pxmin; y1 = (e -> c - x1)/e -> b;};
	if(x2>pxmax)
	    {	x2 = pxmax; y2 = (e -> c - x2)/e -> b;};
	if(x2<pxmin)
	    {	x2 = pxmin; y2 = (e -> c - x2)/e -> b;};
    }	

    else {
	x1 = pxmin;
	if (s1!=(struct Site *)NULL && s1->coord.x > pxmin) 
	    x1 = s1->coord.x;
	if(x1>pxmax) return (0);
	y1 = e -> c - e -> a * x1;
	x2 = pxmax;
	if (s2!=(struct Site *)NULL && s2->coord.x < pxmax) 
	    x2 = s2->coord.x;
	if(x2<pxmin) return(0);
	y2 = e -> c - e -> a * x2;
	if ((y1> pymax & y2>pymax) | (y1<pymin&y2<pymin)) return(0);
	if(y1> pymax)
	    {	y1 = pymax; x1 = (e -> c - y1)/e -> a;};
	if(y1<pymin)
	    {	y1 = pymin; x1 = (e -> c - y1)/e -> a;};
	if(y2>pymax)
	    {	y2 = pymax; x2 = (e -> c - y2)/e -> a;};
	if(y2<pymin)
	    {	y2 = pymin; x2 = (e -> c - y2)/e -> a;};
    }	

    return 1; //a good edge
}
   
void GVoronoi::sitesInit(double coords[], //x0,y0,x2,y2...
			int ns)          //number of sites
{
    freeInit(&sfl, sizeof(Site));

    n_sites = 0;
    sites = new Site[18000];    //arbitrary max-number
    while(n_sites<ns) {
	sites[n_sites].coord.x = coords[n_sites*2];
	sites[n_sites].coord.y = coords[n_sites*2+1];
	sites[n_sites].sitenbr = n_sites+10000;
	n_sites++;
	if(n_sites%18000==0) {
	    cerr<<"too many sites!\n";
	    break;
	}
    }
    qsort(sites, n_sites, sizeof(Site), GVoronoi::scomp);
    xmin = sites[0].coord.x, xmax = sites[0].coord.x;
    for(int i=1; i<n_sites; i++) {
	if(sites[i].coord.x < xmin) xmin = sites[i].coord.x;
	if(sites[i].coord.x > xmax) xmax = sites[i].coord.x;
    }
    ymin = sites[0].coord.y;
    ymax = sites[n_sites-1].coord.y;
    siteidx = 0;
    geomInit();
    plotInit();
}

Site*   GVoronoi::nextOne()
{
    for (;siteidx<n_sites; siteidx+= 1)
	{
	    if (siteidx==0 || sites[siteidx].coord.x!=sites[siteidx-1].coord.x 
		|| sites[siteidx].coord.y!=sites[siteidx-1].coord.y)
		{
		    siteidx += 1;
		    return (&sites[siteidx-1]);
		};
	};
    return( (struct Site *)NULL);
}
    

int GVoronoi::scomp(const void* _s1, const void* _s2)
{
    Site* s1 = (Site*)_s1, *s2=(Site*)_s2;
    if(s1 -> coord.y < s2 -> coord.y) return(-1);
    if(s1 -> coord.y > s2 -> coord.y) return(1);
    if(s1 -> coord.x < s2 -> coord.x) return(-1);
    if(s1 -> coord.x > s2 -> coord.x) return(1);
    return(0);
}
    
void GVoronoi::freeInit(FreeList *fl, int size)
{
    fl -> head = (struct FreeNode *) NULL;
    fl -> nodesize = size;
}

char* GVoronoi::getFree(FreeList* fl)
{
    int i; 
    FreeNode *t;
    if(fl->head == (struct FreeNode *) NULL)
	{
	    t =  (struct FreeNode *) myAlloc(sqrt_nsites * fl->nodesize);
	    GAssert(t);
	    for(i=0; i<sqrt_nsites; i+=1) 	
		makeFree((struct FreeNode *)((char *)t+i*fl->nodesize), fl);
	};
    t = fl -> head;
    fl -> head = (fl -> head) -> nextfree;
    return((char *)t);
}

void GVoronoi::makeFree(FreeNode *curr, FreeList *fl)
{
    curr -> nextfree = fl -> head;
    fl -> head = curr;
}

char * GVoronoi::myAlloc(unsigned n)
{
    char *t;
    if ((t=(char*)malloc(n)) == (char *) 0)
	{    printf("Insufficient memory processing site %d (%d bytes in use)\n",
		     siteidx, total_alloc);
	abort();
	};
    total_alloc += n;

    mem_keeper.append((char*)&t, sizeof(char*));

    return(t);
}

void	GVoronoi::geomInit()
{
    struct VEdge e;
    float sn;

    freeInit(&efl, sizeof e);
    n_vertices = 0;
    n_edges = 0;
    sn = n_sites+4;
    sqrt_nsites = (int) sqrt(sn);
    deltay = ymax - ymin;
    deltax = xmax - xmin;

}
    
Site*	GVoronoi::intersect(HalfEdge *el1, HalfEdge *el2)
{
    VEdge *e1,*e2, *e;
    HalfEdge *el;
    double d, xint, yint;
    int right_of_site;
    Site *v;

    e1 = el1 -> ELedge;
    e2 = el2 -> ELedge;
    if(e1 == (struct VEdge*)NULL || e2 == (struct VEdge*)NULL) 
	return ((struct Site *) NULL);
    if (e1->reg[1] == e2->reg[1]) return ((struct Site *) NULL);
    
    d = e1->a * e2->b - e1->b * e2->a;
    /* printf("intersect: d=%g\n", d); */
    if (-1.0e-17<d && d<1.0e-17) 
	{return ((struct Site *) NULL);};
    
    xint = (e1->c*e2->b - e2->c*e1->b)/d;
    yint = (e2->c*e1->a - e1->c*e2->a)/d;

    if( (e1->reg[1]->coord.y < e2->reg[1]->coord.y) ||
	(e1->reg[1]->coord.y == e2->reg[1]->coord.y &&
	 e1->reg[1]->coord.x < e2->reg[1]->coord.x) )
	{el = el1; e = e1;}
    else	
	{el = el2; e = e2;};
    right_of_site = xint >= e -> reg[1] -> coord.x;
    if ((right_of_site && el -> ELpm == le) ||
	(!right_of_site && el -> ELpm == re)) return ((struct Site *) NULL);
    
    v = (struct Site *) getFree(&sfl);
    v -> refcnt = 0;
    v -> coord.x = xint;
    v -> coord.y = yint;
    return(v);
}
    
float 	GVoronoi::dist(Site* s, Site* t)
{
    double dx,dy;
    dx = s->coord.x - t->coord.x;
    dy = s->coord.y - t->coord.y;
    return(sqrt(dx*dx + dy*dy));
}
    
VEdge*	GVoronoi::bisect(Site* s1, Site* s2)
{
    double dx,dy,adx,ady;
    VEdge *newedge;

    newedge = (struct VEdge *) getFree(&efl);
    
    newedge -> reg[0] = s1;
    newedge -> reg[1] = s2;
    ref(s1); 
    ref(s2);
    newedge -> ep[0] = (struct Site *) NULL;
    newedge -> ep[1] = (struct Site *) NULL;
    
    dx = s2->coord.x - s1->coord.x;
    dy = s2->coord.y - s1->coord.y;
    adx = dx>0 ? dx : -dx;
    ady = dy>0 ? dy : -dy;
    newedge -> c = s1->coord.x * dx + s1->coord.y * dy + (dx*dx + dy*dy)*0.5;
    if (adx>ady)
	{	newedge -> a = 1.0; newedge -> b = dy/dx; newedge -> c /= dx;}
    else
	{	newedge -> b = 1.0; newedge -> a = dx/dy; newedge -> c /= dy;};
    
    newedge -> edgenbr = n_edges;
    //out_bisector(newedge);
    n_edges += 1;
    return(newedge);
}

void	GVoronoi::endpoint(VEdge *e, int lr, Site *s)
{
    double x1, y1, x2, y2;
    e -> ep[lr] = s;
    ref(s);
    if(e -> ep[re-lr]== (struct Site *) NULL) return;
    outputEdge(e, x1, y1, x2, y2);
    addToDiagram(e);

#if 0	//preserve the edges!
    deref(e->reg[le]);
    deref(e->reg[re]);
    makeFree((FreeNode*)e, &efl);
#endif

}

int	GVoronoi::right_of(HalfEdge* el, VPoint *p)
{
    VEdge *e;
    Site *topsite;
    int right_of_site, above, fast;
    double dxp, dyp, dxs, t1, t2, t3, yl;

    e = el -> ELedge;
    topsite = e -> reg[1];
    right_of_site = p -> x > topsite -> coord.x;
    if(right_of_site && el -> ELpm == le) return(1);
    if(!right_of_site && el -> ELpm == re) return (0);

    if (e->a == 1.0)
	{
	    dyp = p->y - topsite->coord.y;
	    dxp = p->x - topsite->coord.x;
	    fast = 0;
	    if ((!right_of_site &e->b<0.0) | (right_of_site&e->b>=0.0) ){
		above = dyp>= e->b*dxp;	
		fast = above;
	    }
	    else{
		above = p->x + p->y*e->b > e-> c;
		if(e->b<0.0) above = !above;
		if (!above) fast = 1;
	    };
	    if (!fast){	
		dxs = topsite->coord.x - (e->reg[0])->coord.x;
		above = e->b * (dxp*dxp - dyp*dyp) <
		    dxs*dyp*(1.0+2.0*dxp/dxs + e->b*e->b);
		if(e->b<0.0) above = !above;
	    };
	}
    else  /*e->b==1.0 */
	{
	    yl = e->c - e->a*p->x;
	    t1 = p->y - yl;
	    t2 = p->x - topsite->coord.x;
	    t3 = yl - topsite->coord.y;
	    above = t1*t1 > t2*t2 + t3*t3;
	};
    return (el->ELpm==le ? above : !above);
}	

    
int	GVoronoi::makeVertex(Site *v)
{
    v -> sitenbr = n_vertices++;
    //out_vertex(v);
    return 0;
}
  
 
int	GVoronoi::ref(Site *v)
{
    v -> refcnt ++;
    return 0;
}
  
int	GVoronoi::deref(Site *v)
{
    v -> refcnt --;
    if (v -> refcnt == 0 ) makeFree((FreeNode*)v, &sfl);
    return 0;
}
   

void	GVoronoi::PQinitialize()
{
    int i; 

    PQcount = 0;
    PQmin = 0;
    PQhashsize = 4 * sqrt_nsites;
    PQhash = (struct HalfEdge *) myAlloc(PQhashsize * sizeof *PQhash);
    for(i=0; i<PQhashsize; i+=1) PQhash[i].PQnext = (struct HalfEdge *)NULL;
}


void  GVoronoi::PQinsert(HalfEdge* he, Site* v, double offset)
{
    struct HalfEdge *last, *next;
    he -> vertex = v;
    ref(v);
    he -> ystar = v -> coord.y + offset;
    last = &PQhash[PQbucket(he)];
    while ((next = last -> PQnext) != (struct HalfEdge *) NULL &&
	   (he -> ystar  > next -> ystar  ||
	    (he -> ystar == next -> ystar && v -> coord.x > next->vertex->coord.x)))
	{
	    last = next;
	}
    he -> PQnext = last -> PQnext; 
    last -> PQnext = he;
    PQcount += 1;
}

void GVoronoi::PQdelete(HalfEdge* he)
{
    struct HalfEdge *last;

    if(he ->  vertex != (struct Site *) NULL){	
	last = &PQhash[PQbucket(he)];
	while (last -> PQnext != he) last = last -> PQnext;
	last -> PQnext = he -> PQnext;
	PQcount -= 1;
	deref(he -> vertex);
	he -> vertex = (struct Site *) NULL;
    };
}

int GVoronoi::PQbucket(HalfEdge* he)
{
    int bucket;

    if	(he->ystar < ymin) bucket = 0;
    else if	(he->ystar >= ymax) bucket = PQhashsize-1;
    else   bucket = (int) ((he->ystar - ymin)/deltay * PQhashsize);
    if (bucket<0) bucket = 0;
    if (bucket>=PQhashsize) bucket = PQhashsize-1 ;
    if (bucket < PQmin) PQmin = bucket;
    return(bucket);
}


int	GVoronoi::PQempty()
{
    return (PQcount == 0);
}

VPoint 	GVoronoi::PQ_min()
{
    VPoint answer;

    while(PQhash[PQmin].PQnext == (HalfEdge *)NULL) {PQmin += 1;};
    answer.x = PQhash[PQmin].PQnext -> vertex -> coord.x;
    answer.y = PQhash[PQmin].PQnext -> ystar;
    return (answer);
}

HalfEdge*	GVoronoi::PQextractmin()
{
    HalfEdge *curr;
    curr = PQhash[PQmin].PQnext;
    PQhash[PQmin].PQnext = curr -> PQnext;
    PQcount -= 1;
    return(curr);
}


void	GVoronoi::ELinitialize()
{
    int i;
    freeInit(&hfl, sizeof **ELhash);
    ELhashsize = 2 * sqrt_nsites;
    ELhash = (struct HalfEdge **) myAlloc ( sizeof *ELhash * ELhashsize);
    for(i=0; i<ELhashsize; i +=1) ELhash[i] = (struct HalfEdge *)NULL;
    ELleftend = HEcreate( (struct VEdge *)NULL, 0);
    ELrightend = HEcreate( (struct VEdge *)NULL, 0);
    ELleftend -> ELleft = (struct HalfEdge *)NULL;
    ELleftend -> ELright = ELrightend;
    ELrightend -> ELleft = ELleftend;
    ELrightend -> ELright = (struct HalfEdge *)NULL;
    ELhash[0] = ELleftend;
    ELhash[ELhashsize-1] = ELrightend;

    diagram_edges = 0;
}   
 
HalfEdge*    GVoronoi::HEcreate(VEdge* e, int pm)
{
    struct HalfEdge *answer;
    answer = (struct HalfEdge *) getFree(&hfl);
    answer -> ELedge = e;
    answer -> ELpm = pm;
    answer -> PQnext = (struct HalfEdge *) NULL;
    answer -> vertex = (struct Site *) NULL;
    answer -> ELrefcnt = 0;
    
    return(answer);
}
    
void	GVoronoi::ELinsert(HalfEdge* lb, HalfEdge* _new)
{
    _new -> ELleft = lb;
    _new -> ELright = lb -> ELright;
    (lb -> ELright) -> ELleft = _new;
    lb -> ELright = _new;
}

    
HalfEdge* GVoronoi::ELgethash(int b)
{
    struct HalfEdge *he;

    if(b<0 || b>=ELhashsize) return((struct HalfEdge *) NULL);
    he = ELhash[b]; 
    if (he == (struct HalfEdge *) NULL || 
	he -> ELedge != (struct VEdge *) DELETED ) return (he);
    
    /* Hash table points to deleted half edge.  Patch as necessary. */
    ELhash[b] = (struct HalfEdge *) NULL;
    if ((he -> ELrefcnt -= 1) == 0) makeFree((FreeNode*)he, &hfl);
    return ((struct HalfEdge *) NULL);
}	    


HalfEdge* GVoronoi::ELleftbnd(VPoint* p)
{
    int i, bucket;
    struct HalfEdge *he;

    /* Use hash table to get close to desired HalfEdge */
    bucket = (int) ( (p->x - xmin)/deltax * ELhashsize );
    if(bucket<0) bucket =0;
    if(bucket>=ELhashsize) bucket = ELhashsize - 1;
    he = ELgethash(bucket);
    if(he == (struct HalfEdge *) NULL)	{   
	for(i=1; 1 ; i += 1)  {	
	    if ((he=ELgethash(bucket-i)) != (struct HalfEdge *) NULL) break;
	    if ((he=ELgethash(bucket+i)) != (struct HalfEdge *) NULL) break;
	};
	totalsearch += i;
    };
    ntry += 1;
    /* Now search linear list of HalfEdges for the corect one */
    if (he==ELleftend  || (he != ELrightend && right_of(he,p))) {
	do {he = he -> ELright;} while (he!=ELrightend && right_of(he,p));
	he = he -> ELleft;
    }
    else 
	do {he = he -> ELleft;} while (he!=ELleftend && !right_of(he,p));

    /* Update hash table and reference counts */
    if(bucket > 0 && bucket <ELhashsize-1){
	if(ELhash[bucket] != (struct HalfEdge *) NULL) 
	    ELhash[bucket] -> ELrefcnt -= 1;
	ELhash[bucket] = he;
	ELhash[bucket] -> ELrefcnt += 1;
    };	
    return (he);
}
    
void	GVoronoi::ELdelete(HalfEdge* he)
{
    (he -> ELleft) -> ELright = he -> ELright;
    (he -> ELright) -> ELleft = he -> ELleft;
    he -> ELedge = (struct VEdge *)DELETED;
}
    
HalfEdge* GVoronoi::ELright(HalfEdge* he)
{
    return (he -> ELright);
}
    
HalfEdge* GVoronoi::ELleft(HalfEdge* he)
{
    return he->ELleft;
}

Site*	GVoronoi::leftreg(HalfEdge* he)
{
    if(he -> ELedge == (struct VEdge *)NULL) return(bottomsite);
    return( he -> ELpm == le ? 
	    he -> ELedge -> reg[le] : he -> ELedge -> reg[re]);
}
    
Site*	GVoronoi::rightreg(HalfEdge * he)
{
    if(he -> ELedge == (struct VEdge *)NULL) return(bottomsite);
    return( he -> ELpm == le ? 
	    he -> ELedge -> reg[re] : he -> ELedge -> reg[le]);
}

void	GVoronoi::compute()
{
    struct Site *newsite, *bot, *top, *temp, *p;
    struct Site *v;
    struct VPoint newintstar;
    int pm;
    struct HalfEdge *lbnd, *rbnd, *llbnd, *rrbnd, *bisector;
    struct VEdge *e;


    PQinitialize();
    bottomsite = nextOne();
    //out_site(bottomsite);
    ELinitialize();

    newsite = nextOne();
    while(1){
	if(!PQempty()) newintstar = PQ_min();

	if (newsite != (struct Site *)NULL 
	    && (PQempty() 
		|| newsite -> coord.y < newintstar.y
		|| (newsite->coord.y == newintstar.y 
		    && newsite->coord.x < newintstar.x)))
	    {/* new site is smallest */
		//out_site(newsite);
		lbnd = ELleftbnd(&(newsite->coord));
		rbnd = ELright(lbnd);
		bot = rightreg(lbnd);
		e = bisect(bot, newsite);
		bisector = HEcreate(e, le);
		ELinsert(lbnd, bisector);
		if ((p = intersect(lbnd, bisector)) != (struct Site *) NULL) 
		{	PQdelete(lbnd);
			PQinsert(lbnd, p, dist(p,newsite));
		};
		lbnd = bisector;
		bisector = HEcreate(e, re);
		ELinsert(lbnd, bisector);
		if ((p = intersect(bisector, rbnd)) != (struct Site *) NULL)
		{	PQinsert(bisector, p, dist(p,newsite));	
		};
		newsite = nextOne();
	    }
	else if (!PQempty()) 
	/* intersection is smallest */
	    {	lbnd = PQextractmin();
		llbnd = ELleft(lbnd);
		rbnd = ELright(lbnd);
		rrbnd = ELright(rbnd);
		bot = leftreg(lbnd);
		top = rightreg(rbnd);
		//out_triple(bot, top, rightreg(lbnd));
		v = lbnd->vertex;
		makeVertex(v);
		endpoint(lbnd->ELedge,lbnd->ELpm,v);
		endpoint(rbnd->ELedge,rbnd->ELpm,v);
		ELdelete(lbnd); 
		PQdelete(rbnd);
		ELdelete(rbnd); 
		pm = le;
		if (bot->coord.y > top->coord.y)
		{	temp = bot; bot = top; top = temp; pm = re;}
		e = bisect(bot, top);
		bisector = HEcreate(e, pm);
		ELinsert(llbnd, bisector);
		endpoint(e, re-pm, v);
		deref(v);
		if((p = intersect(llbnd, bisector)) != (struct Site *) NULL)
		{	PQdelete(llbnd);
			PQinsert(llbnd, p, dist(p,bot));
		};
		if ((p = intersect(bisector, rrbnd)) != (struct Site *) NULL)
		{	PQinsert(bisector, p, dist(p,bot));
		};
	    }
	else break;
    };

    double x1, y1, x2, y2;
    for(lbnd=ELright(ELleftend); lbnd != ELrightend; lbnd=ELright(lbnd))
	{
	    e = lbnd -> ELedge;
	    outputEdge(e, x1, y1, x2, y2);
	    addToDiagram(e);
	};
}

    
