/****************************************************************************
 *   gmmap.cpp : 
 *   		implementation of portable mmap() stuff.
 *
 *   Change Log :
 *		 June 16, 1998 Initial version,  Liujian Qian
 *
 *   $Id: gmmap.cpp,v 1.9 1998/12/02 04:03:50 qian Exp $
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

#include "gcxx.h"
#include "gmmap.h"

GMMap::GMMap(const char* f)
{
    if(strlen(f) >255) {
	msgWarn("invalid file name!");
    }
    
    strcpy(file, f);

    fd = -1;
    mapped = 0;
    protection = PROT_READ | PROT_WRITE; 	//default: rw
    flag = MAP_SHARED;			//default: shared
    offset = last_offset = 0;
    map_len = last_map_len = 0;

    start_hint = map_region = last_map_region = (void*) 0;
}

GMMap::~GMMap()
{
    unmap();
    if(fd>=0) ::close(fd);
}

int GMMap::map(off_t off, size_t len, const char* ps)
{
    if(ps) setProtection(ps);

    if(fd<0) {
	if((protection & PROT_READ) && (protection & PROT_WRITE))
	    fd = ::open(file, O_RDWR);
	else if( (protection & PROT_READ) )
	    fd = ::open(file, O_RDONLY);
	else if( (protection & PROT_WRITE) )
	    fd = ::open(file, O_WRONLY);
	if(fd<0) {
	    msgWarn("can't open file for mapping.");
	    return 0;
	}
    }

    if(mapped) unmap();
    
    last_map_len = map_len, last_offset = offset;
    map_len  = len, offset = off;	
    last_map_region = map_region;
    map_region = mmap(NULL, map_len, protection, flag, fd, offset);
    
    return mapped = (map_region==MAP_FAILED)?0:1;
}

int GMMap::mapAll(const char* ps)
{
    struct stat s;
    ::stat(file, &s);
    long sz = s.st_size;
    
    return map(0, sz, ps);
}    
    
int GMMap::extendMap(int size_inc, long off, long len)
{
    cout<<"increasing by "<<size_inc<<endl;
    
    unmap();
    last_map_len = map_len; 
    if(len>0) { //if map length is explicitly given
	map_len  = len;
    }
    else map_len += size_inc;   //default is to increase the map length
    
    if(off>0) {	//new offset is given
	last_offset = offset;
	offset = off;	
    }
    
    last_map_region = map_region;   
    ::lseek(fd, size_inc-1, SEEK_END);
    ::write(fd, "\0", 1);
    map_region = mmap(NULL, map_len, protection, flag, fd, offset);
    return mapped = (map_region==MAP_FAILED)?0:1;
}

int GMMap::unmap()
{
    mapped = 0;
    if(munmap((char*)map_region, map_len) <0) {
	msgWarn("error in unmapping: invalid region or length?");
	return 0;
    } else return 1;
}

int GMMap::advise(int )
{
    //do nothing here    
#if 0
    madvise(map_region, map_len, ad);
#endif
    return 1;
}

void GMMap::setFlag(const char* fs) 
{
    int f=0;
    if(strchr(fs, 's')) f |= MAP_SHARED;
    if(strchr(fs, 'p')) f |= MAP_PRIVATE;
#ifdef _SUNOS_
    if(strchr(fs, 'n')) f |= MAP_NORESERVE;
#endif
    if(strchr(fs, 'f')) f |= MAP_FIXED;
    flag = f;
}

void GMMap::setProtection(const char* ps)
{
    int p =0;
    if(strchr(ps, 'r')) p |= PROT_READ;
    if(strchr(ps, 'w')) p |= PROT_WRITE;
    if(strchr(ps, 'x')) p |= PROT_EXEC;
    if(strchr(ps, 'n')) p |= PROT_NONE;
    protection = p;
}

void* GMMap::ptr() 
{
    if(!mapped) return (void*) 0;
    return (void*) (map_region!=MAP_FAILED)?map_region : 0;
}

int  GMMap::len() const 
{
    return mapped? map_len:0;
}

