/****************************************************************************
 *   gshmem.cpp : 
 *   		implementation of GShmem class.
 *
 *   Change Log :
 *		 Mar. 16 1998 Initial version,  Liujian Qian
 *
 *   $Id: gshmem.cpp,v 1.1.1.1 1998/05/21 05:37:27 qian Exp $
 ***************************************************************************/
#include "gshmem.h"
#include <string.h>

Result
GShmem::create(uint32 sz, mode_t mode)
{
    destroy();
    _size = sz;

    if ((_id = shmget(IPC_PRIVATE, int(_size), IPC_CREAT|mode)) == -1)  {
	msgFail("can't get shared memory");
    }

    if ((_base = (char*)shmat(_id, 0, 0)) == (char*)-1) {
	_base = 0;
	msgFail("can't attach to shared memory");
    }

    return GOk;
}    


Result
GShmem::destroy()
{
    if (_base)  {
	detach();
	shmctl(_id, IPC_RMID, 0);
	_base = 0;
    }

    return GOk;
}

Result
GShmem::set(uid_t uid,  gid_t	gid, mode_t mode)
{
    shmid_ds buf;

    if(uid)  buf.shm_perm.uid = uid;
    if(gid)  buf.shm_perm.gid = gid;
    if(mode) buf.shm_perm.mode = mode;

    if (shmctl(_id, IPC_SET, &buf) == -1) {
	return GFail; // error
    }
    _size = buf.shm_segsz;
    return GOk;
}

Result
GShmem::attach(int i)
{
    _id = i;

    if ((_base = (char*) shmat(_id, 0, 0)) == (char*)-1) 
	msgFail("can't attach to existing shared memory ");

    shmid_ds buf;
    if (shmctl(_id, IPC_STAT, &buf) == -1)
	return GFail;

    _size = buf.shm_segsz;

    return GOk;
}

Result
GShmem::detach()
{
    if((_base != NULL) && (shmdt(_base) < 0)) 
	msgFail("can't detach shared memory");

    return GOk;
}


void
GShmem::clean()
{
    if(_base) {
	memset(_base, 0, _size);
    }
}

void
GShmem::setIntValue(int off, int v)
{
    
    if(_base && off<_size && off>=0) {
	int * command = (int*) (_base + off);
	*command = v;
    }
}

	
