/****************************************************************************
 *   gxmlwf.cpp : 
 *   		implementation of an XML processor.
 *
 *   Change Log :
 *		 May 14 1998 Initial version,  Liujian Qian
 *
 *   $Id: gxmlwf.cpp,v 1.9 1999/01/27 21:00:02 qian Exp $
 ***************************************************************************/

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>

#include "gxmlwf.h"

GXMLAttr::GXMLAttr()
{
    value = 0, next = 0, owner = 0;
}

GXMLAttr::GXMLAttr(const char* _name, const char* val)
{
    strcpy(name, _name);
    value = new char[strlen(val)+1];
    strcpy(value, val);
    next = 0, owner = 0;
}

GXMLAttr::~GXMLAttr()
{
    if(value) delete[] value;
}
    
void GXMLAttr::set(const char* _name, const char* _value)
{
    strcpy(name, _name);
    int len = strlen(_value) + 1;
    value = new char[len];
    strcpy(value, _value);
}

bool GXMLAttr::operator==(const GXMLAttr& att) 
{
    return strcmp(name, att.name)==0?TRUE:FALSE;
}

bool GXMLAttr::operator==(const char* _name) 
{
    return strcmp(_name, name)==0?TRUE:FALSE;
}

ostream&  operator<<(ostream& o, const GXMLAttr& a)
{
    return o<<" "<<a.name<<"=\""<<a.value<<"\"";
}

//////////////////////////////////////////////////////////////////////
GXMLElement::GXMLElement()
{
    parent = children = next = 0;
    attrs = 0;
    n_children = n_attr = 0;
}

GXMLElement::~GXMLElement()
{
    //simply delete *children and *next; which will roll down
    if(children) {
	delete children;
	children = 0;
    }
    if(next) {
	delete next;
	next = 0;
    }

    /* ??? */
    if(attrs) {
	delete attrs;
	attrs = 0;
    }
}
    
void GXMLElement::addChild(GXMLElement* el)
{
    GXMLElement *c=children;
    if(!c) children = el;
    else {
	while(c->next) c=c->next;
	c->next = el;
    }
    n_children++;
}

void GXMLElement::addAttr(GXMLAttr* att)
{
    GXMLAttr *a = attrs;
    
    if(!a) attrs = att;
    else {
	while(a->next)  a=a->next; 
	a->next = att;
    }
    n_attr ++;
}

void GXMLElement::setNext(GXMLElement* el)
{
    next = el;
}

void GXMLElement::setParent(GXMLElement* el)
{
    parent = el;
}

bool GXMLElement::isLeaf() const
{
    if(!children) return TRUE;
    else return FALSE;
}

bool GXMLElement::isAllSpace()
{
    const char* p = content.ptr();
    for(int i=0; i<content.size(); i++) {
	if(!isspace(p[i])) return FALSE;
    }
    return TRUE;
}

const char* GXMLElement::getAttributeValue(const char* attr)
{
    GXMLAttr* att = attrs;
    while(att) {
	if(strcmp(attr, att->name)==0) return (const char*) att->value;
	att = att->next;
    }
    return 0;
}

bool GXMLElement::operator==(const GXMLElement& el) 
{
    return strcmp(name, el.name)==0?TRUE:FALSE;
}
bool GXMLElement::operator==(const char* _name) 
{
    return strcmp(_name, name)==0?TRUE:FALSE;
}

ostream& operator<<(ostream& o, const GXMLElement& a)
{
    o<<"\n<"<<a.name;
    GXMLAttr *att = a.attrs;
    while(att) {
	o<<*att;
	att = att->next;
    }
    o<<">";
    const char* p=a.content.ptr();
    for(int i=0; i<a.content.size(); i++) o<<p[i];
    o<<"</"<<a.name;
    return o<<">";
}


/**
 * the  handler functions
 */
static void XCData(void *usr_data, const char* s, int len)
{
    GXMLElementStk* stk = (GXMLElementStk*) usr_data;    
    GXMLElement* cur = (GXMLElement*) stk->top();
    assert(cur);
    if(len>0) {
	cur->content.append(s, len);
    }
}

static int attCmp(const void *att1, const void *att2)
{
    return strcmp(*(const char **)att1, *(const char **)att2);
}

static void XStartElement(void *usr_data, const char* name, const char** atts)
{
    int n_atts;
    const char **p, *n, *v;
    GXMLElementStk* stk = (GXMLElementStk*) usr_data;
    GXMLElement *parent, *ne;
    ne = new GXMLElement;
    strcpy(ne->name, name);
    parent = (GXMLElement*) stk->top();
    ne->setParent(parent);
    if(parent) parent->addChild(ne);

    // set-up its attributes
    p = atts;
    while (*p)
    ++p;
    n_atts = (p - atts) >> 1;
    if(n_atts > 1) 
	qsort((void*)atts, n_atts, sizeof(char*)*2, attCmp);
    while(*atts) {
	n = *atts++; 
	v = *atts++;	
	ne->addAttr( new GXMLAttr(n, v) );
    }
    
    // push current element into stack
    stk->push(ne);  
}
    
static void XEndElement(void *user_data, const char *)
{
    GXMLElementStk* stk = (GXMLElementStk*) user_data;
    GXMLElement* el = (GXMLElement*) stk->top();
    
    if(el->isAllSpace()) el->content.recycle();
	
    if(stk->size() >1) 
	stk->pop();
    else {   //!!! levae the root element in the stack !!!
	//GXMLElement *el = stk->top();
    }
}

static void XProcInstruction(void * /*user_data*/, 
			     const char * /*target*/, 
			     const char * /*data*/)
{

}

//////////////////////////////////////////////////////////////////////
GXMLProcessor::GXMLProcessor(const char* xml)
{
    stk = new GXMLElementStk;
    strcpy(xml_file, xml);

    encoding = (const char*)0;
    root = (GXMLElement*)0;

    parser = XML_ParserCreate(encoding);
    
    XML_SetUserData(parser, stk);    
    XML_SetElementHandler(parser, XStartElement, XEndElement);
    XML_SetCharacterDataHandler(parser, XCData);
    XML_SetProcessingInstructionHandler(parser, XProcInstruction);
}


GXMLProcessor::~GXMLProcessor()
{
    if(stk)  delete stk;
    if(root) delete root;
    
    XML_ParserFree(parser);
}

int GXMLProcessor::parseStream()
{
    int fd = ::open(xml_file, O_RDONLY);
    if (fd < 0) {
	cerr<<"can't open "<<xml_file<<endl;
	return 0;
    }
    for (;;) {
	int nread;
	char *buf = (char*) XML_GetBuffer(parser, READ_SIZE);
	if (!buf) {
	    ::close(fd);
	    cerr<<xml_file<<": out of memory\n";
	    return 0;
	}
	nread = ::read(fd, buf, READ_SIZE);
	if (nread < 0) {
	    cerr<<"can't open "<<xml_file<<endl;
	    ::close(fd);
	    return 0;
	}
	if (!XML_ParseBuffer(parser, nread, nread == 0)) {
	    reportError();
	    close(fd);
	    return 0;
	}
	if (nread == 0) {
	    close(fd);
	    break;;
	}	
    }

    //retrieve the root element
    root = (GXMLElement*) stk->pop();
    
    return 1;
}  
    

void GXMLProcessor::printAll() const
{
    if(root) {
	recursivePrintElement(cout,0, root);
	cout<<"\n";
    }
}

int GXMLProcessor::recursivePrintElement(ostream& o, 
					 int level, 
					 const GXMLElement* el) const
{
    if(!el) return 0;

    o<<"\n";
    for(int i=0; i<level; i++) o<<"\t";
    o<<"<"<<el->name;
    GXMLAttr *att = el->attrs;
    while(att) {
	o<<*att;
	att = att->next;
    }
    o<<">";
    if(el->content.size()) {
	const char* p = el->content.ptr();
	for(int i=0; i<el->content.size(); i++) o<<p[i];
    }
    
    GXMLElement* child = el->children;
    GXMLElement* next = el->next;
    recursivePrintElement(o, level+1, child);
    if(el->content.size()==0) { //align elements with no conents
	o<<"\n";
	for(int i=0; i<level; i++) o<<"\t";
    }
    o<<"</"<<el->name;
    o<<">";

    return recursivePrintElement(o, level, next);
}

void GXMLProcessor::reportError()
{
    int code = XML_GetErrorCode(parser);
    const char *message = XML_ErrorString(code);
    if (message) {
	cerr<<xml_file<<":"<<XML_GetErrorLineNumber(parser);
	cerr<<":"<<XML_GetErrorColumnNumber(parser);
	cerr<<":"<<message<<"\n";
    }
    else
	cerr<<xml_file<<": (unknown message "<<code<<")\n";
}
 
int GXMLProcessor::writeStream(const char* file)
{
    ofstream os(file);
    time_t t = time((time_t*)NULL);
    char *buf;
    buf = ctime(&t);

    os<<"<?xml version=\"1.0\" ?>\n";
    os<<"<!-- =+= This file is generated by GFC. \n";
    os<<"     =+= Localtime= "<<buf;
    os<<"     =+= Modify with caution.\n";
    os<<"-->\n";
    recursivePrintElement(os,0, root);

    return 1;
}


GXMLElement* GXMLProcessor::_search(const char* key, GXMLElement* el) 
{
    GXMLElement* e;

    if(!el) return 0;
    
    //found the (first) one that matches the key. stop traversing.
    if(strcmp(el->name, key)==0) return el;

    GXMLElement* child = el->children;
    GXMLElement* next = el->next;
    if( (e=_search(key, next)) ) return e;
    else  if( (e=_search(key, child)) ) return e;
    else  return (GXMLElement*) 0;
}
    
GXMLElement* GXMLProcessor::search(const char* key)
{
    if(!root) {
	msgWarn("XML file not parsed yet!");
	return 0;
    }
    
    GXMLElement* el = _search(key, root);
    if(el) {
	return el;
    }
    else return 0;
}

    
